# Generated by Django 2.2 on 2019-12-27 12:25

import aparnik.contrib.messaging.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Messaging',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('source_display_name', models.CharField(max_length=150, null=True)),
                ('action', models.CharField(max_length=50)),
                ('category', models.CharField(max_length=50)),
                ('obj', models.IntegerField(blank=True, null=True)),
                ('url', models.URLField(blank=True, null=True)),
                ('short_description', models.CharField(max_length=100)),
                ('channels', aparnik.contrib.messaging.fields.ListField(max_length=200)),
                ('extra_data', aparnik.contrib.messaging.fields.JSONField(default={})),
                ('is_read', models.BooleanField(default=False)),
                ('create_date', models.DateTimeField(auto_now_add=True)),
                ('update_date', models.DateTimeField(auto_now=True)),
                ('recipient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL)),
                ('source', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-id',),
            },
        ),
    ]



from django.conf.urls import url

from aparnik.contrib.province.api.views import TownCreateAPIView, TownDetailAPIView, TownListAPIView

app_name = 'province'

urlpatterns = [
    url(r'^$', TownListAPIView.as_view(), name='list'),
    url(r'^create/$', TownCreateAPIView.as_view(), name='create'),
    url(r'^(?P<id>\d+)/$', TownDetailAPIView.as_view(), name='detail')
]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-01 11:42


from django.db import migrations



def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    File = apps.get_model('files', 'File')
    # File = apps.get_model('filefields', 'FileFields')
    for file in File.objects.all():
        if file.password and file.password != '':
            file.file_obj.password = file.password
            file.file_obj.iv = file.iv
            file.file_obj.is_encrypt_needed = True
            file.file_obj.save()


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    File = apps.get_model('files', 'File')
    for file in File.objects.all():
        if file.file_obj.is_encrypt_needed:
            file.password = file.file_obj.password
            file.iv = file.file_obj.iv
            file.save()


class Migration(migrations.Migration):

    dependencies = [
        ('files', '0005_file_publish_date'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-04-28 16:36


from django.db import migrations


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    BaseCourse = apps.get_model('courses', 'BaseCourse')
    try:
        for course in BaseCourse.objects.all():
            course.content = course.description
            course.save()
    except Exception:
        pass


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    BaseCourse = apps.get_model('courses', 'BaseCourse')
    try:
        for course in BaseCourse.objects.all():
            course.description = course.content
            course.save()
    except Exception:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0012_basecourse_content'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-26 12:12


import aparnik.contrib.users.models
from django.db import migrations
import django_enumfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0026_auto_20190526_1210'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='devicelogin',
            name='device_type_temp',
        ),
        migrations.AlterField(
            model_name='devicelogin',
            name='device_type',
            field=django_enumfield.db.fields.EnumField(default=0, enum=aparnik.contrib.users.models.DeviceType, verbose_name='Device Type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('books', '0003_auto_20181026_1131'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='book',
            options={'verbose_name': '\\u06a9\\u062a\\u0627\\u0628', 'verbose_name_plural': '\\u06a9\\u062a\\u0627\\u0628 \\u0647\\u0627'},
        ),
        migrations.AlterModelOptions(
            name='booksegment',
            options={'verbose_name': '\\u0628\\u062e\\u0634 \\u06a9\\u062a\\u0627\\u0628', 'verbose_name_plural': '\\u0628\\u062e\\u0634 \\u06a9\\u062a\\u0627\\u0628 \\u0647\\u0627'},
        ),
        migrations.AlterModelOptions(
            name='publisher',
            options={'verbose_name': '\\u0645\\u0646\\u062a\\u0634\\u0631 \\u06a9\\u0646\\u0646\\u062f\\u0647', 'verbose_name_plural': '\\u0645\\u0646\\u062a\\u0634\\u0631 \\u06a9\\u0646\\u0646\\u062f\\u0647'},
        ),
        migrations.AlterModelOptions(
            name='writertranslator',
            options={'verbose_name': '\\u0645\\u062a\\u0631\\u062c\\u0645', 'verbose_name_plural': '\\u0645\\u062a\\u0631\\u062c\\u0645\\u06cc\\u0646'},
        ),
        migrations.AlterField(
            model_name='book',
            name='category_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='categories.Category', verbose_name='\\u062f\\u0633\\u062a\\u0647 \\u0628\\u0646\\u062f\\u06cc'),
        ),
        migrations.AlterField(
            model_name='book',
            name='published_date',
            field=models.DateTimeField(verbose_name='\\u062a\\u0627\\u0631\\u06cc\\u062e \\u0627\\u0646\\u062a\\u0634\\u0627\\u0631'),
        ),
        migrations.AlterField(
            model_name='book',
            name='publisher_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='books.Publisher', verbose_name='\\u0645\\u0646\\u062a\\u0634\\u0631 \\u06a9\\u0646\\u0646\\u062f\\u0647'),
        ),
        migrations.AlterField(
            model_name='book',
            name='sample_book',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='\\u06a9\\u062a\\u0627\\u0628 \\u0646\\u0645\\u0648\\u0646\\u0647'),
        ),
        migrations.AlterField(
            model_name='book',
            name='translators_obj',
            field=models.ManyToManyField(blank=True, related_name='library_translators', to='books.WriterTranslator', verbose_name='\\u0645\\u062a\\u0631\\u062c\\u0645\\u06cc\\u0646'),
        ),
        migrations.AlterField(
            model_name='book',
            name='writers_obj',
            field=models.ManyToManyField(related_name='library_writers', to='books.WriterTranslator', verbose_name='\\u0646\\u0648\\u06cc\\u0633\\u0646\\u062f\\u0647 \\u0647\\u0627'),
        ),
        migrations.AlterField(
            model_name='publisher',
            name='title',
            field=models.CharField(max_length=255, verbose_name='\\u0639\\u0646\\u0648\\u0627\\u0646'),
        ),
        migrations.AlterField(
            model_name='writertranslator',
            name='first_name',
            field=models.CharField(max_length=60, verbose_name='\\u0646\\u0627\\u0645'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.contrib.admin.options import StackedInline, TabularInline
from django.apps import apps

from jalali_date.admin import ModelAdminJalaliMixin
from dynamic_raw_id.admin import DynamicRawIDMixin

from aparnik.utils.utils import is_app_installed
if is_app_installed('aparnik.contrib.managements'):
    from aparnik.contrib.managements.models import ManagementActions, FieldList
from .models import BaseModel


# from questionsanswer.models import QA, Like

class BaseAdmin(ModelAdminJalaliMixin, DynamicRawIDMixin, admin.ModelAdmin):
    fields = []
    list_filter = []
    list_display = []
    search_fields = []
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []


class BaseModelStackedInline(StackedInline, DynamicRawIDMixin):
    exclude = ['review_average', 'visit_count']
    dynamic_raw_id_fields = []


class BaseModelTabularInline(TabularInline, DynamicRawIDMixin):
    exclude = ['review_average', 'visit_count']
    dynamic_raw_id_fields = []


class BaseModelAdmin(BaseAdmin):
    fields = []
    list_display = ['id', ]
    list_filter = []
    search_fields = []
    exclude = ['review_average', 'update_needed']
    raw_id_fields = []
    dynamic_raw_id_fields = []
    readonly_fields = []
    allowed_actions = []
    save_as = True

    def __init__(self, *args, **kwargs):
        Klass = BaseModelAdmin
        Klass_parent = BaseAdmin

        super(Klass, self).__init__(*args, **kwargs)
        if issubclass(self.model, BaseModel):
            self.fields = self.fields + ['is_show_only_for_super_user', 'tags', 'sort']
            self.list_display = self.list_display + ['sort']

        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = BaseModel

    def get_queryset(self, request):
        qs = super(BaseModelAdmin, self).get_queryset(request)
        if not request.user.is_superuser:
            if issubclass(self.model, BaseModel):
                return qs.filter(is_show_only_for_super_user=False)
        return qs

    def get_action_choices(self, request, default_choices=[]):
        if not is_app_installed('aparnik.contrib.managements') or request.user.is_superuser:
            return super(BaseModelAdmin, self).get_action_choices(request,
                                                                  default_choices=self.allowed_actions)

        # if request.user.is_superuser:
        #     return super(BaseModelAdmin, self).get_action_choices(request,
        #                                                           default_choices=self.allowed_actions) + self.get_allowed_actions(
        #         request)
        else:
            return self.get_allowed_actions(request)

    def get_form(self, request, obj=None, **kwargs):
        form = super(BaseModelAdmin, self).get_form(request, obj=obj, **kwargs)

        if not is_app_installed('aparnik.contrib.managements'):
            return form

        editable_fields = self.get_editable_fields(self, request)
        allowed_fields = self.get_allowed_permissions(request)
        if editable_fields == 'all':
            return form
        else:
            try:
                if obj:
                    for field in allowed_fields:
                        if field not in editable_fields:
                            form.base_fields[field].disabled = True
                if obj is None:
                    for field in allowed_fields:
                        if field not in editable_fields:
                            form.base_fields[field].disabled = True
            except:
                pass

        return form

    def get_fieldsets(self, request, obj=None):
        fieldsets = super(BaseModelAdmin, self).get_fieldsets(request, obj=obj)
        for fieldset in fieldsets:
            if 'is_show_only_for_super_user' in fieldset[1]['fields']:
                if type(fieldset[1]['fields']) == tuple:
                    fieldset[1]['fields'] = list(fieldset[1]['fields'])
                fieldset[1]['fields'].remove('is_show_only_for_super_user')
            if 'tags' in fieldset[1]['fields']:
                if type(fieldset[1]['fields']) == tuple:
                    fieldset[1]['fields'] = list(fieldset[1]['fields'])
                fieldset[1]['fields'].remove('tags')


        if not is_app_installed('aparnik.contrib.managements'):
            return fieldsets

        allowed_fields = self.get_allowed_permissions(request)

        if allowed_fields == 'all':
            return fieldsets
        else:
            field_sets = []

            for permits in fieldsets:
                for permit in permits[1]['fields']:
                    if permit in allowed_fields:
                        field_sets.append(permit)

            return [(None, {'fields': field_sets})]

    def get_list_display(self, request, obj=None):
        if not is_app_installed('aparnik.contrib.managements'):
            return super(BaseModelAdmin, self).get_list_display(request)

        allowed_fields = self.get_allowed_permissions(request)

        if allowed_fields == 'all':
            return super(BaseModelAdmin, self).get_list_display(request)
        else:
            list_display = []

            for permit in super(BaseModelAdmin, self).get_list_display(request):
                if permit in allowed_fields:
                    list_display.append(permit)

        return list_display

    def get_list_filter(self, request, obj=None):
        if not is_app_installed('aparnik.contrib.managements'):
            return super(BaseModelAdmin, self).get_list_filter(request)

        allowed_fields = self.get_allowed_permissions(request)

        if allowed_fields == 'all':
            return super(BaseModelAdmin, self).get_list_filter(request)
        else:
            list_filter = []

            for permit in super(BaseModelAdmin, self).get_list_filter(request):
                if permit in allowed_fields:
                    list_filter.append(permit)

        return list_filter

    # این قطعه کد کار می کنه ولی بدلیل پیچیدگی که داره مثلا در سفارشات کلا سرچ فیلد خالی میشه
    # def get_search_fields(self, request, obj=None):
    #     allowed_fields = self.get_allowed_permissions(request)
    #
    #     if allowed_fields == 'all':
    #         return super(BaseModelAdmin, self).get_search_fields(request)
    #     else:
    #         search_fields = []
    #
    #         for permit in super(BaseModelAdmin, self).get_search_fields(request):
    #             if permit in allowed_fields:
    #                 search_fields.append(permit)
    #
    #     return search_fields

    def get_allowed_permissions(self, request, model_name=None, active=True):
        if not is_app_installed('aparnik.contrib.managements'):
            return
        group = list(request.user.groups.values_list('id', flat=True).all())
        current_app = request.resolver_match.func.model_admin.model._meta.app_label
        if not model_name:
            model_name = request.resolver_match.func.model_admin.model._meta.object_name

        view_permissions = list(
            FieldList.objects.filter(management__group__in=group, group_id__in=group,
                                     management__application=current_app, model=model_name,
                                     is_enable=active
                                     , permission__title__contains='view').values_list('name', flat=True))

        if request.user.is_superuser:
            view_permissions = 'all'

        return view_permissions

    @staticmethod
    def get_editable_fields(self, request):
        if not is_app_installed('aparnik.contrib.managements'):
            return
        group = list(request.user.groups.values_list('id', flat=True).all())
        current_app = request.resolver_match.func.model_admin.model._meta.app_label
        model_name = request.resolver_match.func.model_admin.model._meta.object_name

        edit_permissions = list(
            FieldList.objects.filter(management__group__in=group, management__application=current_app, model=model_name,
                                     is_enable=True
                                     , permission__title__contains='edit').values_list('name', flat=True))

        if request.user.is_superuser:
            edit_permissions = 'all'

        return edit_permissions

    def get_allowed_actions(self, request):
        if not is_app_installed('aparnik.contrib.managements'):
            return
        group = list(request.user.groups.values_list('id', flat=True).all())
        current_app = request.resolver_match.func.model_admin.model._meta.app_label
        model_name = request.resolver_match.func.model_admin.model._meta.object_name

        actions = list(
            ManagementActions.objects.filter(fieldlist__management__group__in=group, fieldlist__model=model_name,
                                             fieldlist__management__application=current_app).values_list(
                'title', 'description'))

        return actions

    # Control what field to show in an admin page inline
    def get_formsets_with_inlines(self, request, obj=None):
        for inline in self.get_inline_instances(request, obj):
            if not request.user.is_superuser and is_app_installed('aparnik.contrib.managements'):
                # hide MyInline in the add view
                allowed_fields = self.get_allowed_permissions(request, inline.model._meta.object_name)
                field_sets = []
                model_fields = [item.name for item in inline.model._meta.get_fields() if
                                not item.auto_created and item.concrete and item.editable]

                if not inline.fields:
                    inline.fields = []

                for permit in model_fields:
                    if permit in allowed_fields and permit in inline.fields:
                        field_sets.append(permit)

                if inline.exclude:
                    for f in inline.exclude:
                        if f in field_sets:
                            field_sets.remove(f)
                inline.fields = field_sets
            yield inline.get_formset(request, obj), inline

    # Handling save procedure for management deactivated fields
    def save_formset(self, request, form, formset, change):
        if not is_app_installed('aparnik.contrib.managements') or request.user.is_superuser:
            return super(BaseModelAdmin, self).save_formset(request, form, formset, change)

        for inline in self.get_inline_instances(request):
            instances = formset.save(commit=False)
            group = list(request.user.groups.values_list('id', flat=True).all())
            current_app = request.resolver_match.func.model_admin.model._meta.app_label
            model_name = formset.model._meta.object_name

            allowed_fields = self.get_allowed_permissions(request, inline.model._meta.object_name)
            field_sets = []
            model_fields = [item.name for item in inline.model._meta.get_fields() if
                            not item.auto_created and item.concrete and item.editable]

            for permit in model_fields:
                if permit not in allowed_fields:
                    field_sets.append(permit)
            try:
                for instance in instances:
                    for field in field_sets:
                        default = FieldList.objects.filter(name=field, management__group__in=group,
                                                           model=model_name,
                                                           management__application=current_app).values_list(
                            'default',
                            flat=True).first()
                        if default:
                            try:
                                # Check if it is a foreign key
                                getattr(instance, field + '_id')
                                field_obj = instance._meta.get_field(field).remote_field.model.objects.filter(
                                    id=default).first()
                                # Entered default value is not correct
                                if not field_obj:
                                    raise ValueError(
                                        '{' + str(field) + " default value not found for foreign key" + '}')

                                # Save default value
                                setattr(instance, field + '_id', field_obj.id)
                                instance.save()
                            except:
                                # it is not a related field so reaplace it as text
                                setattr(instance, field, default)
                                instance.save()
            except:
                raise ValueError('{' + str(field) + " default value not found" + '}')

        return super(BaseModelAdmin, self).save_formset(request, form, formset, change)



    def save_model(self, request, obj, form, change):
        if not is_app_installed('aparnik.contrib.managements'):
            return super(BaseModelAdmin, self).save_model(request, obj, form, change)

        group = list(request.user.groups.values_list('id', flat=True).all())
        current_app = request.resolver_match.func.model_admin.model._meta.app_label
        model_name = request.resolver_match.func.model_admin.model._meta.object_name

        request_object = apps.get_model(current_app, model_name)

        allowed_fields = self.get_fieldsets(request)[0][1]['fields']
        obj.allowed_permissions = self.get_allowed_permissions(request)

        if not request.user.is_superuser:

            # model_fields = [x.name for x in request_object._meta.get_fields(
            #     include_parents=True) if
            #                 hasattr(x,
            #                         'blank') and x.blank == False and 'ptr' not in x.name and 'ctype' not in x.name and x.default]

            model_fields = []

            for x in request_object._meta.get_fields(include_parents=True):
                attr_value = False
                try:
                    attr_value = True if getattr(obj, x.name) else False
                except:
                    attr_value = False

                if hasattr(x, 'blank') and \
                        x.blank == False and \
                        'ptr' not in x.name and \
                        'ctype' not in x.name and \
                        x.default and \
                        not attr_value:
                    model_fields.append(x.name)

            request_object = obj
            filtered_fields = [field for field in model_fields if field not in allowed_fields]

            for field in filtered_fields:

                ##Enable this line to fetch expected value from field's own table in django model
                # default = request.resolver_match.func.model_admin.model._meta.model.objects.filter().values_list(field,flat=True).first()

                # Checking field kind
                is_fk = request.resolver_match.func.model_admin.model._meta.get_field(field).is_relation
                m2m = request.resolver_match.func.model_admin.model._meta.get_field(field).many_to_many
                one2one = request.resolver_match.func.model_admin.model._meta.get_field(field).one_to_one
                field_type = request.resolver_match.func.model_admin.model._meta.get_field(field).get_internal_type()

                default = FieldList.objects.filter(name=field, management__group__in=group, model=model_name,
                                                   management__application=current_app).values_list('default',
                                                                                                    flat=True).first()

                try:
                    if (one2one or is_fk or m2m):
                        field_obj = (request_object._meta.get_field(field)).remote_field.model.objects.filter(
                            pk=default).first()
                        if default:
                            setattr(request_object, (field + '_pk'), field_obj.pk)

                    elif field_type not in ['AutoField']:
                        setattr(request_object, field, default)

                    obj = request_object

                except:
                    raise ValueError('{' + str(field) + " default value not found" + '}')

        super(BaseModelAdmin, self).save_model(request, obj, form, change)



from rest_framework import serializers

from aparnik.contrib.basemodels.models import BaseModel


class BaseModelAdminSerializer(serializers.ModelSerializer):
    sort_count = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BaseModelAdminSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseModel
        fields = [
            'id',
            'sort_count',
        ]

        read_only_fields = [
            'id',
            'sort_count',
        ]

    def get_sort_count(self, obj):
        return obj.sort_count if hasattr(obj, 'sort_count') else 0



# Generated by Django 2.2 on 2019-12-27 12:58

import aparnik.contrib.chats.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('filefields', '0013_auto_20190714_1639'),
    ]

    operations = [
        migrations.CreateModel(
            name='ChatSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('update_at', models.DateTimeField(auto_now=True)),
                ('uri', models.URLField(default=aparnik.contrib.chats.models._generate_unique_uri)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ChatSessionMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('update_at', models.DateTimeField(auto_now=True)),
                ('message', models.TextField(default='', max_length=2000)),
                ('chat_session', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='messages', to='chats.ChatSession')),
                ('file_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='File')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ChatSessionMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('update_at', models.DateTimeField(auto_now=True)),
                ('chat_session', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='members', to='chats.ChatSession')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.conf.urls import url, include

from .views import OrderSortAdminAPIView, OrderAdminListAPIView

app_name = 'orders'

urlpatterns = [
    url(r'^$', OrderAdminListAPIView.as_view(), name='list'),
    url(r'^sort/$', OrderSortAdminAPIView.as_view(), name='sort'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-04-28 16:44


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0013_auto_20190428_1636'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='basecourse',
            name='description',
        ),
    ]



"""
Django settings for src project.

Generated by 'django-admin startproject' using Django 1.8.4.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import dj_database_url
import datetime
from django.utils.translation import ugettext_lazy as _

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get('DJANGO_SECRET_KEY')
if not SECRET_KEY:
    raise ValueError('Secret key does not set')

INSTALLED_APPS = (
    'aparnik',
    'aparnik.contrib.suit',
    'debug_toolbar',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'jalali_date',  # required for admin jalali date
    'rest_framework',
    'fcm_django',
    'dynamic_raw_id',
    'django_filters',
    'ckeditor',
    'ckeditor_uploader',
    'tagulous',
    'colorfield',
    # This applications
    'rangefilter',
    'import_export',
    'channels',

    'aparnik.contrib.province',
    'aparnik.contrib.users',
    'aparnik.contrib.audits',
    'aparnik.contrib.addresses',
    'aparnik.contrib.bankaccounts',
    'aparnik.contrib.managements',
    'aparnik.contrib.aboutus',
    'aparnik.contrib.contactus',
    'aparnik.contrib.invitation',
    'aparnik.contrib.basemodels',
    'aparnik.contrib.filefields',
    'aparnik.contrib.bookmarks',
    'aparnik.contrib.reviews',
    'aparnik.contrib.questionanswers',
    'aparnik.contrib.socials',
    'aparnik.contrib.sliders',
    'aparnik.contrib.settings',
    'aparnik.contrib.counters',
    'aparnik.contrib.notifications',
    'aparnik.contrib.messaging',
    'aparnik.contrib.chats',
    'aparnik.contrib.notifiesme',
    'aparnik.contrib.shortblogs',
    'aparnik.contrib.supports',
    'aparnik.contrib.faq',
    'aparnik.contrib.termsandconditions',
    'aparnik.contrib.categories',
    'aparnik.contrib.pages',
    'aparnik.contrib.segments',
    'aparnik.contrib.buttons',
    'aparnik.contrib.tickets',
    'aparnik.packages.shops.products',
    'aparnik.packages.shops.productssharing',
    'aparnik.packages.bankgateways.zarinpals',
    'aparnik.packages.shops.payments',
    'aparnik.packages.shops.subscriptions',
    'aparnik.packages.shops.orders',
    'aparnik.packages.shops.coupons',
    'aparnik.packages.shops.cosales',
    'aparnik.packages.shops.vouchers',
    'aparnik.packages.shops.files',
    'aparnik.packages.educations.educations',
    'aparnik.packages.educations.books',
    'aparnik.packages.educations.teachers',
    'aparnik.packages.educations.courses',
    'aparnik.packages.educations.progresses',
    'aparnik.packages.news',

)

MIDDLEWARE = (
    # 'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sites.middleware.CurrentSiteMiddleware',
    # third party
    # 'DJANGO_PROJECT.middleware.WebFactionFixes',
    'DJANGO_PROJECT.middleware.DisableCSRFMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.middleware.locale.LocaleMiddleware',
)
# ROOT_URLCONF = 'DJANGO_PROJECT.urls.common_urls'
ROOT_URLCONF = 'DJANGO_PROJECT.urls'
ASGI_APPLICATION = "DJANGO_PROJECT.routing.application"

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')]
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'debug': True,
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                # 'applications.context_processors.application',
                # 'cart.context_processors. cart',
            ],
        },
    },
]

# WSGI_APPLICATION = 'DJANGO_PROJECT.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db2.sqlite3'),
    }
}

FILE_UPLOAD_HANDLERS = [
    'django.core.files.uploadhandler.TemporaryFileUploadHandler',
    'django.core.files.uploadhandler.MemoryFileUploadHandler',
]

# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/


LANGUAGE_CODE = 'fa-ir'  # 'en-us'
# LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'Asia/Tehran'

USE_I18N = True

USE_L10N = True

USE_TZ = os.environ.get('USE_TZ', "True") == "True"

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale/'),
)

LANGUAGES = [
    ('fa', _('Persian')),
    # ('en', _('English')),
    # ('es', _('Espaniol')),
]

# Django Rest Framework

REST_FRAMEWORK = {
    # Attention: change it for custom handle
    # 'EXCEPTION_HANDLER': 'users.utils.custom_exception_handler',
    'DEFAULT_PERMISSION_CLASSES': (
        # 'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_RENDERER_CLASSES': (
        # Attention: change it for custom json response
        # 'users.customJSON.CustomJSONRenderer',
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.BrowsableAPIRenderer',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework_jwt.authentication.JSONWebTokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    ),
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'DEFAULT_FILTER_BACKENDS': ('django_filters.rest_framework.DjangoFilterBackend',),
    'PAGE_SIZE': 25,
}

# Allow all host headers
ALLOWED_HOSTS = ['*']

# STATICFILES_DIRS = (
#     os.path.join(BASE_DIR, 'static'),
# )


JWT_AUTH = {
    'JWT_ENCODE_HANDLER':
        'rest_framework_jwt.utils.jwt_encode_handler',

    'JWT_DECODE_HANDLER':
        'rest_framework_jwt.utils.jwt_decode_handler',

    'JWT_PAYLOAD_HANDLER':
        'rest_framework_jwt.utils.jwt_payload_handler',

    'JWT_PAYLOAD_GET_USER_ID_HANDLER':
        'rest_framework_jwt.utils.jwt_get_user_id_from_payload_handler',

    'JWT_RESPONSE_PAYLOAD_HANDLER':
    # 'users.api.views.jwt_response_payload_handler',
        'aparnik.contrib.users.api.views.jwt_response_payload_handler',
    # 'rest_framework_jwt.utils.jwt_response_payload_handler',
    # 'my_jwt.views.jwt_response_payload_handler',

    'JWT_SECRET_KEY': SECRET_KEY,
    'JWT_GET_USER_SECRET_KEY': None,
    'JWT_PUBLIC_KEY': None,
    'JWT_PRIVATE_KEY': None,
    'JWT_ALGORITHM': 'HS256',
    'JWT_VERIFY': True,
    'JWT_VERIFY_EXPIRATION': True,
    'JWT_LEEWAY': 0,
    'JWT_EXPIRATION_DELTA': datetime.timedelta(days=300),
    'JWT_AUDIENCE': None,
    'JWT_ISSUER': None,

    'JWT_ALLOW_REFRESH': False,
    'JWT_REFRESH_EXPIRATION_DELTA': datetime.timedelta(days=7),

    'JWT_AUTH_HEADER_PREFIX': 'APARNIK',
    'JWT_AUTH_COOKIE': None,

}

# Django debug toolbar
DEBUG_TOOLBAR_PATCH_SETTINGS = True
INTERNAL_IPS = ('127.0.0.1',)

CORS_ORIGIN_ALLOW_ALL = True

# CORS_ORIGIN_WHITELIST = (
#     'restzahedipro.s3.amazonaws.com',
# )
# CORS_ORIGIN_ALLOW_ALL = True
# Access-Control-Allow-Origin: *

# jalali date time
JALALI_FORMAT = os.environ.get('JALALI_FORMAT', "%Y/%m/%d")

APARNIK = {
    'AWS_ACTIVE': os.environ.get('AWS_ACTIVE', "False") == "True",
    'BANK_ACTIVE': os.environ.get('BANK_ACTIVE', "False") == "True",
    'ZARINPAL_MERCHENT_CODE': os.environ.get('ZARINPAL_MERCHENT_CODE', ''),
    # 'USER_UPDATE_SERIALIZER': 'users.api.serializers.UserUpdateSerializer',
    'USER_LOGIN_WITH_PASSWORD': os.environ.get('USER_LOGIN_WITH_PASSWORD', "True") == "True",
    # 'USER_LOGIN_API_VIEW': 'users.api.views.UserLoginAPIView',
    # 'USER_IS_SHOW_ADMIN_PANEL': False,
    'USER_REGISTER_WITH_LOGIN': os.environ.get('USER_REGISTER_WITH_LOGIN', "True") == "True",
    'API_PRODUCT_MODE': os.environ.get('DJANGO_API_PRODUCT_MODE', "True") == "True",
    'SMS_EXPIRE_TOKEN': os.environ.get('SMS_EXPIRE_TOKEN', 180),
    'SMS_API_KEY': os.environ.get('SMS_API_KEY', ''),
    'SMS_OTA_NAME': os.environ.get('SMS_OTA_NAME', ""),
    'SEND_SMS': os.environ.get('SEND_SMS', 'True') == 'True',
    'VERIFY_FAKE': os.environ.get('VERIFY_FAKE', 'False') == 'True',
    'VERIFY_FAKE_CODE': os.environ.get('VERIFY_FAKE_CODE', ''),
    'JALALI_FORMAT': JALALI_FORMAT,
}

AWS_ACTIVE = APARNIK['AWS_ACTIVE']
USER_LOGIN_WITH_PASSWORD = APARNIK['USER_LOGIN_WITH_PASSWORD']

CKEDITOR_CONFIGS = {
    'basic': {
        'toolbar': 'Basic',
    },
    'advance': {
        'toolbar': 'Advance',
    },
    'custom': {
        'toolbar': 'Custom',
        'toolbar_Custom': [
            {'name': 'clipboard', 'items': ['Cut', 'Copy', 'Paste', 'PasteText', 'PasteFromWord', '-', 'Undo', 'Redo']},
            {'name': 'editing', 'items': ['Find', 'Replace', '-', 'SelectAll']},
            {'name': 'colors', 'items': ['TextColor', 'BGColor']},
            {'name': 'styles', 'items': ['Styles', 'Format', 'Font', 'FontSize']},
            {'name': 'forms',
             'items': ['Form', 'Checkbox', 'Radio', 'TextField', 'Textarea', 'Select', 'Button', 'ImageButton',
                       'HiddenField']},
            {'name': 'basicstyles',
             'items': ['Bold', 'Italic', 'Underline', 'Strike', 'Subscript', 'Superscript', '-', 'RemoveFormat']},
            {'name': 'paragraph',
             'items': ['NumberedList', 'BulletedList', '-', 'Outdent', 'Indent', '-', 'Blockquote', 'CreateDiv', '-',
                       'JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock', '-', 'BidiLtr', 'BidiRtl',
                       'Language']},
            {'name': 'links', 'items': ['Link', 'Unlink', 'Anchor']},
            {'name': 'insert',
             'items': ['Table', 'HorizontalRule', 'Smiley', 'SpecialChar']},  # 'Image', 'Flash','PageBreak', 'Iframe'
        ]
    }
}

CKEDITOR_UPLOAD_PATH = "/"

TAGGIT_CASE_INSENSITIVE = True

# User
# AUTH_USER_MODEL = 'users.User'
# AUTHENTICATION_BACKENDS = ('aparnik.contrib.users.backends.AuthBackend', 'django.contrib.auth.backends.ModelBackend')
AUTH_USER_MODEL = 'aparnik_users.User'
AUTHENTICATION_BACKENDS = (
    'aparnik.contrib.users.backends.AuthBackend',
    # 'django.contrib.auth.backends.ModelBackend'
)

NOTIFICATION_API_KEY = os.environ.get('NOTIFICATION_API_KEY', None)
NOTIFICATION_APP_ID = os.environ.get('NOTIFICATION_APP_ID', None)
FCM_DJANGO_SETTINGS = {
    'FCM_SERVER_KEY': NOTIFICATION_API_KEY,
    # true if you want to have only one active device per registered user at a time
    # default: False
    'ONE_DEVICE_PER_USER': False,
    # devices to which notifications cannot be sent,
    # are deleted upon receiving error response from FCM
    # default: False
    'DELETE_INACTIVE_DEVICES': False,
}

TINYMCE_DEFAULT_CONFIG = {
    'height': 360,
    'width': 1120,
    'cleanup_on_startup': True,
    'custom_undo_redo_levels': 20,
    'selector': 'textarea',
    'theme': 'modern',
    'plugins': 'textcolor save link image media preview codesample contextmenu table code lists fullscreen  insertdatetime  nonbreaking contextmenu directionality searchreplace wordcount visualblocks visualchars code fullscreen autolink lists  charmap print  hr anchor pagebreak',
    'toolbar1': 'save fullscreen bold italic underline,fontselect,fontsizeselect  | forecolor backcolor | alignleft aligncenter alignright alignjustify'
                '| bullist numlist | outdent indent | table | link image media | codesample | preview code |',
    'contextmenu': 'formats | link image',

    'menubar': True,
    'statusbar': True,

}

# Filebrowser
FILEBROWSER_DIRECTORY = ''
DIRECTORY = ''

# notifications settings
NOTIFICATIONS_CHANNELS = {
    'websocket': 'aparnik.contrib.messaging.channels.BasicWebSocketChannel',
    'push': 'aparnik.contrib.messaging.channels.BasicPushNotificationChannel',
}


CELERY_BROKER_URL = 'redis://localhost:6379'
CELERY_RESULT_BACKEND = 'redis://localhost:6379'

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels_redis.core.RedisChannelLayer",
        "CONFIG": {
            "hosts": [("localhost", 6379)],
        },
    },
}

SMS_CHANNEL = 'aparnik.utils.sms_api.SMSAPIKavenegar'



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta

from aparnik.packages.shops.payments.models import Payment, Order


class Command(BaseCommand):
    # Show this when the user types help
    help = "check payments"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        # one_hours = now() - relativedelta(hours=1)
        # current_time = now()

        payments = Payment.objects.active().filter(status=Payment.STATUS_COMPLETE,
                                                   order_obj__status=Order.query_waiting(),
                                                   # update_at__range=[one_hours, current_time]
                                                   )
        for payment in payments:
            payment.order_obj.pay_success()
        finished_time = now()

        print(('check payments %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-14 17:42


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('buttons', '0004_auto_20181125_2227'),
    ]

    operations = [
        migrations.AddField(
            model_name='button',
            name='title_color',
            field=models.CharField(blank=True, max_length=6, null=True, verbose_name='Title color'),
        ),
    ]



"""
WSGI config for testproject project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/howto/deployment/wsgi/
"""

import os
import django

from channels.routing import get_default_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DJANGO_PROJECT.settings")
django.setup()
application = get_default_application()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-06 16:00


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sliders', '0003_remove_slider_pages'),
    ]

    operations = [
        migrations.CreateModel(
            name='SliderImage',
            fields=[
                ('slider_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='sliders.Slider')),
            ],
            options={
                'verbose_name': 'Image Slide Show',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Image Slide Shows',
            },
            bases=('sliders.slider',),
        ),
    ]



from django.db.models import Q
from rest_framework.permissions import ( AllowAny, IsAuthenticated )
from rest_framework.generics import ListAPIView

from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from ..models import Subscription


class SubscriptionListAPIView(ListAPIView):
    serializer_class = ModelListPolymorphicSerializer
    permission_classes = [IsAuthenticated]
    # filter_backends = (filters.SearchFilter,)

    def get_queryset(self, *args, **kwargs):
        if 'product_id' in self.request.query_params:
            product_id = self.request.query_params['product_id']
            return Subscription.objects.active().filter(
                Q(products__in=[product_id]) |
                Q(type=Subscription.TYPE_ALL
                  )
            ).distinct()
        return Subscription.objects.active()



from django.urls import reverse
from rest_framework import serializers

from aparnik.contrib.basemodels.api.serializers import BaseModelListSerializer, BaseModelDetailSerializer
from ..models import Payment


class PaymentSummarySerializer(BaseModelListSerializer):
    date = serializers.SerializerMethodField()
    method = serializers.SerializerMethodField()

    class Meta:
        model = Payment
        fields = BaseModelListSerializer.Meta.fields + [
            'date',
            'method',
            'bank_reference',
            'status',
            'uuid',
        ]

    def get_date(self, obj):

        return obj.update_at if obj.update_at else obj.created_at

    def get_method(self, obj):

        return obj.get_method_json_display()


    # request.build_absolute_uri(payment.get_url())

class PaymentDetailSerializer(BaseModelDetailSerializer, PaymentSummarySerializer):

    url_pay = serializers.SerializerMethodField()
    url_pay_cancel = serializers.SerializerMethodField()
    date = serializers.SerializerMethodField()
    method = serializers.SerializerMethodField()
    order = serializers.SerializerMethodField()

    class Meta:
        model = Payment
        fields = PaymentSummarySerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [
            'url_pay',
            'url_pay_cancel',
            'date',
            'method',
            'bank_reference',
            'status',
            'uuid',
            'order',
        ]

    def get_url_pay(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_url())

    def get_url_pay_cancel(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_pay_cancel_uri())

    def get_order(self, obj):
        from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
        order_summary = ModelListPolymorphicSerializer(obj.order_obj, many=False, read_only=True, context=self.context)
        return order_summary.data



from rest_framework import serializers

from aparnik.contrib.users.models import User
from aparnik.contrib.users.api.serializers import UserSummaryListSerializer


class UserAdminListSerializer(UserSummaryListSerializer):
    id = serializers.SerializerMethodField()
    sort_count = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(UserAdminListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = User
        fields = UserSummaryListSerializer.Meta.fields + [
            'id',
            'sort_count',
        ]

        read_only_fields = UserSummaryListSerializer.Meta.read_only_fields + [
            'id',
            'sort_count',
        ]

    def get_id(self, obj):
        return obj.pk

    def get_sort_count(self, obj):
        return obj.sort_count if hasattr(obj, 'sort_count') else 0



from rest_framework import permissions
from rest_framework.permissions import SAFE_METHODS


class IsAdminPermission(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has an `owner` attribute.
    """

    def has_object_permission(self, request, view, obj):
        if request.user and request.user.is_admin:
            return True
        return False



# Generated by Django 2.2 on 2020-03-16 11:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0013_auto_20190615_1430'),
        ('courses', '0021_auto_20190714_1639'),
    ]

    operations = [
        migrations.CreateModel(
            name='CourseFileSegment',
            fields=[
                ('basesegment_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='segments.BaseSegment')),
            ],
            options={
                'verbose_name': 'Course File Segment',
                'verbose_name_plural': 'Course Files Segments',
            },
            bases=('segments.basesegment',),
        ),
    ]



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta
from django.core.management import call_command

User = get_user_model()


class Command(BaseCommand):
    # Show this when the user types help
    help = "handle update"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        # handle course
        call_command('courseindexing')

        call_command('reviewssummary')

        call_command('progresssummary')

        call_command('notifiesme')

        call_command('sendnotifications')

        call_command('checkpayments')

        call_command('modelvisitupdate')

        finished_time = now()

        print(('handle objects update %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))



import os
from celery import Celery
from django.conf import settings

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'DJANGO_PROJECT.settings')

app = Celery('DJANGO_PROJECT')
app.config_from_object('django.conf:settings', namespace='CELERY')
app.conf.task_default_queue = os.path.basename(os.path.normpath(settings.BASE_DIR))
app.autodiscover_tasks()


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-02-15 19:16


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bankaccounts', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bankaccount',
            name='shaba_number',
            field=models.CharField(max_length=255, unique=True, verbose_name='Shaba Number'),
        ),
    ]



from django.shortcuts import get_object_or_404
from django.utils.text import Truncator
from rest_framework import serializers
from django.contrib.auth import get_user_model

from aparnik.contrib.users.api.serializers import UserSummaryListSerializer
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer

from ..models import ChatSession, ChatSessionMessage, ChatSessionMember, ChatSessionTypeEnum, ChatMessageNotification

User = get_user_model()


# serializer
# Chat Session
class ChatSessionListSerializer(serializers.ModelSerializer):
    url = serializers.SerializerMethodField()
    url_members = serializers.SerializerMethodField()
    url_messages = serializers.SerializerMethodField()
    url_messages_create = serializers.SerializerMethodField()
    url_mark_as_read = serializers.SerializerMethodField()
    owner = serializers.SerializerMethodField()
    title_session = serializers.SerializerMethodField()
    cover = serializers.SerializerMethodField()
    unread_count = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    class Meta:
        model = ChatSession
        fields = [
            'id',
            'url',
            'url_members',
            'url_messages',
            'url_messages_create',
            'url_mark_as_read',
            'owner',
            'title_session',
            'cover',
            'uri',
            'type',
            'unread_count',
            'created_at',
            'resourcetype',
        ]

        read_only_fields = [
            'id',
            'url',
            'url_members',
            'url_messages',
            'url_messages_create',
            'url_mark_as_read',
            'uri',
            'title_session',
            'cover',
            'unread_count',
            'created_at',
            'resourcetype',
        ]


    def get_owner(self, obj):
        return UserSummaryListSerializer(obj.owner, many=False, read_only=True, context=self.context).data

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_url_members(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_member_api_uri())

    def get_url_messages(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_messages_api_uri())

    def get_url_messages_create(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_messages_create_api_uri())

    def get_url_mark_as_read(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_mark_as_read_uri())

    def get_title_session(self, obj):
        if obj.type == ChatSessionTypeEnum.PRIVATE:
            user = self.context['request'].user
            if user == obj.owner:
                member = obj.members.first()
                if member:
                    return member.user.get_full_name()
            return obj.owner.get_full_name()
        return obj.title

    def get_cover(self, obj):
        cover_obj = None
        if obj.type == ChatSessionTypeEnum.PRIVATE:
            cover_obj = obj.owner.avatar
            user = self.context['request'].user
            if user == obj.owner:
                member = obj.members.first()
                if member:
                    cover_obj = member.user.avatar
        else:
            cover_obj = obj.cover_obj
        if cover_obj:
            return FileFieldListSerailizer(cover_obj, many=False, read_only=True, context=self.context).data
        return None

    def get_resourcetype(self, obj):
        return obj.resourcetype

    def get_unread_count(self, obj):
        return ChatMessageNotification.objects.unread_count_chat_session(self.context['request'].user, obj)


# Details Serializer
class ChatSessionDetailsSerializer(ChatSessionListSerializer):
    # members
    username_mention = serializers.CharField(write_only=True, required=False)
    title = serializers.CharField(max_length=255, write_only=True, required=False)
    cover_obj_id = serializers.IntegerField(write_only=True, required=False)

    class Meta:
        model = ChatSession
        fields = ChatSessionListSerializer.Meta.fields + [
            'username_mention',
            'title',
            'cover_obj_id',
        ]

        read_only_fields = ChatSessionListSerializer.Meta.read_only_fields + [

        ]

    def update(self, instance, validated_data):
        self.handle_username_mention(instance, validated_data)
        return super(ChatSessionDetailsSerializer, self).update(instance, validated_data)
    
    def create(self, validated_data):
        username_mention = validated_data.get('username_mention', None)
        if username_mention:
            del validated_data['username_mention']
        instance = super(ChatSessionDetailsSerializer, self).create(validated_data)
        validated_data['username_mention'] = username_mention
        self.handle_username_mention(instance, validated_data)
        return instance

    def handle_username_mention(self, instance, validated_data):
        username_mention = validated_data.get('username_mention', None)
        if username_mention is not None:
            user = get_object_or_404(User.objects.all(), username_mention=username_mention)

            owner = instance.owner

            if owner != user:  # Only allow non owners join the room
                instance.members.get_or_create(
                    user=user, chat_session=instance
                )

    # def get_members(self, obj):
    #     members = [
    #         UserSummaryListSerializer(chat_session.user, many=False, read_only=True, context={'request': request})
    #         for chat_session in chat_session.members.all()
    #     ]
    #     members.insert(0, owner)  # Make the owner the first member

# Chat session member
class ChatSessionMemberListSerializer(serializers.ModelSerializer):
    user = serializers.SerializerMethodField()
    url_delete = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    class Meta:
        model = ChatSessionMember
        fields = [
            'user',
            'url_delete',
            'resourcetype',
        ]

        read_only_fields = [
            'user',
            'url_delete',
            'resourcetype',
        ]

    def get_user(self, obj):
        return UserSummaryListSerializer(obj.user, many=False, read_only=True, context=self.context).data

    def get_url_delete(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_resourcetype(self, obj):
        return obj.resourcetype


class ChatSessionMemberDetailsSerializer(ChatSessionMemberListSerializer):

    class Meta:
        model = ChatSessionMember
        fields = ChatSessionMemberListSerializer.Meta.fields + [

        ]

        read_only_fields = ChatSessionMemberListSerializer.Meta.read_only_fields + [

        ]


# ChatSessionMessage
class ChatSessionMessageListSerializer(serializers.ModelSerializer):
    file = serializers.SerializerMethodField()
    user = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()
    
    class Meta:
        model = ChatSessionMessage
        fields = [
            'id',
            'user',
            'message',
            'file',
            'resourcetype',
            'created_at',
        ]

        read_only_fields = [
            'file',
            'resourcetype',
            'created_at',
        ]

    def get_file(self, obj):
        if obj.file_obj:
            return FileFieldListSerailizer(obj.file_obj, many=False, read_only=True, context=self.context).data
        return None

    def get_user(self, obj):
        return UserSummaryListSerializer(obj.user, many=False, read_only=True, context=self.context).data

    def get_resourcetype(self, obj):
        return obj.resourcetype

# Details Serializer
class ChatSessionMessageDetailsSerializer(ChatSessionMessageListSerializer):

    file_obj_id = serializers.IntegerField(required=False, write_only=True)

    class Meta:
        model = ChatSessionMessage
        fields = ChatSessionMessageListSerializer.Meta.fields + [
            'file_obj_id',
        ]

        read_only_fields = ChatSessionMessageListSerializer.Meta.read_only_fields + [

        ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from .models import Product, ProductProperty, ProductPropertyMembership


# Register your models here.
class ProductPropertyAdmin(BaseModelAdmin):
    fields = ['title', 'icon', ]
    list_display = ['title', ]
    search_fields = ['title', ]
    list_filter = []
    exclude = []
    raw_id_fields = ['icon']
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = ProductPropertyAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = ProductProperty


class ProductPropertyMembershipInline(BaseModelTabularInline):
    model = Product.properties.through
    fields = ['property', 'value']
    raw_id_fields = ('property',)
    extra = 1
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = ProductPropertyMembershipInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude



class ProductAdmin(BaseModelAdmin):
    fields = ['title', 'further_details', 'price_fabric', 'slider_segment_obj', 'delivery_type',
              'is_free_field', 'discount_percent_value', 'discount_percent_expire',
              'is_discount_percent_expire_show',
              'aparnik_bon_return_value',
              'aparnik_bon_return_expire_value',
              'maximum_use_aparnik_bon_value',
              'has_permit_use_wallet_value',
              'currency',
              'is_tax',
              'is_draft',
              ]
    list_display = ['title', 'currency', 'price_fabric', 'is_free_field', 'discount_percent_value', 'is_draft',]
    search_fields = ['title', 'price_fabric', 'discount_percent_value']
    list_filter = ['is_free_field', 'is_draft',]
    exclude = []
    raw_id_fields = []
    inlines = [ProductPropertyMembershipInline]
    dynamic_raw_id_fields = ['slider_segment_obj', ]

    def __init__(self, *args, **kwargs):
        Klass = ProductAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Product

    def get_fieldsets(self, request, obj=None):
        fieldsets = super(ProductAdmin, self).get_fieldsets(request, obj)
        # if not obj:
        #     return fieldsets
        def remove_key(key):
            if key in fieldset[1]['fields']:
                if type(fieldset[1]['fields']) == tuple:
                    fieldset[1]['fields'] = list(fieldset[1]['fields'])
                fieldset[1]['fields'].remove(key)

        if not request.user.is_superuser:  # or request.user.pk == obj.pk:
            # fieldsets = deepcopy(fieldsets)
            for fieldset in fieldsets:
                remove_key('currency')
                remove_key('is_tax')
                remove_key('delivery_type')


        return fieldsets

admin.site.register(Product, ProductAdmin)
admin.site.register(ProductProperty, ProductPropertyAdmin)



from ..base import *

DEBUG = os.environ.get('DJANGO_DEBUG', "False") == "True"
# Application definition
INSTALLED_APPS += (
    #third party apps
    's3direct',
    #my apps
)

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/

# Update database configuration with $DATABASE_URL.
if os.environ.get('HEROKU_ACTIVE', "False") == "False":
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': os.environ.get('DATABASE_NAME'),
            'USER': os.environ.get('DATABASE_USER'),
            'PASSWORD': os.environ.get('DATABASE_PASSWORD'),
            'HOST': 'localhost',
            'PORT': '',
        }
    }

db_from_env = dj_database_url.config(conn_max_age=500)
DATABASES['default'].update(db_from_env)
# Honor the 'X-Forwarded-Proto' header for request.is_secure()
# SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')

# Amazon
AWS_HEADERS = {  # see http://developer.yahoo.com/performance/rules.html#expires
        'Expires': 'Thu, 31 Dec 2099 20:00:00 GMT',
        'Cache-Control': 'max-age=94608000',
}

AWS_STORAGE_BUCKET_NAME = os.environ.get('AWS_STORAGE_BUCKET_NAME')
AWS_ACCESS_KEY_ID = os.environ.get('AWS_ACCESS_KEY_ID')
AWS_SECRET_ACCESS_KEY = os.environ.get('AWS_SECRET_ACCESS_KEY')

if not AWS_SECRET_ACCESS_KEY or not AWS_ACCESS_KEY_ID or not AWS_STORAGE_BUCKET_NAME:
        raise ValueError('AWS KEY does not set')

S3DIRECT_REGION = os.environ.get('S3DIRECT_REGION', 'us-east-1')

if AWS_ACTIVE:
    # Tell django-storages that when coming up with the URL for an item in S3 storage, keep
    # it simple - just use this domain plus the path. (If this isn't set, things get complicated).
    # This controls how the `static` template tag from `staticfiles` gets expanded, if you're using it.
    # We also use it in the next setting.
    # AWS_S3_CUSTOM_DOMAIN = '%s.s3.amazonaws.com' % AWS_STORAGE_BUCKET_NAME
    AWS_S3_CUSTOM_DOMAIN = 's3-%s.amazonaws.com/%s' % (S3DIRECT_REGION, AWS_STORAGE_BUCKET_NAME)
    # This is used by the `static` template tag from `static`, if you're using that. Or if anything else
    # refers directly to STATIC_URL. So it's safest to always set it.
    # #
    STATICFILES_LOCATION = 'static'
    STATICFILES_STORAGE = 'custom_storages.StaticStorage'
    STATIC_URL = "https://%s/%s/" % (AWS_S3_CUSTOM_DOMAIN, STATICFILES_LOCATION)
    # #
    # STATIC_ROOT = os.path.join(BASE_DIR, 'staticfiles')
    # STATIC_URL = '/static/'

    # MEDIA
    MEDIAFILES_LOCATION = 'media'
    MEDIA_URL = "https://%s/%s/" % (AWS_S3_CUSTOM_DOMAIN, MEDIAFILES_LOCATION)
    DEFAULT_FILE_STORAGE = 'custom_storages.MediaStorage'
else:
    MEDIA_URL = "/media/"
    MEDIA_ROOT = os.path.join(os.path.dirname(BASE_DIR), "media")

    STATIC_ROOT = os.path.join(os.path.dirname(BASE_DIR), 'static')
    STATIC_URL = '/static/'


def create_filename(filename):
    import uuid
    ext = filename.split('.')[-1]
    filename = '%s.%s' % (uuid.uuid4().hex, ext)
    return os.path.join('files', filename)

def create_file_filename(filename):
    import uuid
    ext = filename.split('.')[-1]
    filename = '%s.%s' % (uuid.uuid4().hex, ext)
    return os.path.join('file', filename)


S3DIRECT_DESTINATIONS = {
    # Allow anybody to upload any MIME type
    # 'misc': {
    #     'key': '/'
    # },

    # Allow staff users to upload any MIME type
    # 'pdfs': {
    #     'key': 'uploads/pdfs',
    #     'auth': lambda u: u.is_staff
    # },

    # Allow anybody to upload jpeg's and png's. Limit sizes to 500b - 4mb
    'images': {
        # 'key': 'uploads/images',
        'key': create_filename,
        'auth': lambda u: u.is_authenticated,
        'allowed': [
            'image/jpeg',
            'image/png'
        ],
        'content_length_range': (500, 4000000),
    },

    # Allow authenticated users to upload mp4's
    'videos': {
        # 'key': 'uploads/videos',
        'key': create_filename,
        'auth': lambda u: u.is_authenticated,
        'allowed': ['video/mp4']
    },

    'file': {
        # 'key': 'uploads/videos',
        'key': create_file_filename,
        'auth': lambda u: u.is_authenticated,
        # TODO: add mime type
        # 'allowed': ['application/octet-stream ipa']
    },
    # Allow anybody to upload any MIME type with a custom name function
    # 'custom_filename': {
    #     'key': create_filename
    # },
}


# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Count, Q, F, Sum
from django.db.models.functions import Coalesce
from polymorphic.models import PolymorphicManager, PolymorphicModel
from django.utils.translation import ugettext as _
from django.urls import reverse
from django.contrib.auth import get_user_model

import tagulous

from aparnik.utils.utils import human_format, field_with_prefix

User = get_user_model()


# BaseModel MANAGER
class Tag(tagulous.models.TagModel):
    class TagMeta:
        # Tag options
        # initial = "eating, coding, gaming"
        # force_lowercase = True
        pass

    def get_api_uri(self):
        return reverse('aparnik-api:models:tags-details', args=[self.id])


class BaseModelManager(PolymorphicManager):

    def get_queryset(self):
        return super(BaseModelManager, self).get_queryset()

    def active(self, user=None):
        return self.get_queryset()

    def update_needed(self):
        return self.active().filter(update_needed=True)


# BaseModel class
class BaseModel(PolymorphicModel):
    review_average = models.FloatField(default=0.0, verbose_name=_('Review Average'))

    tags = tagulous.models.TagField(to=Tag, blank=True)
    sort = models.IntegerField(default=0, verbose_name=_('Sort'))
    visit_count = models.IntegerField(default=0, verbose_name=_('Visit counter'))
    update_needed = models.BooleanField(default=False, verbose_name=_('Update needed'))
    is_show_only_for_super_user = models.BooleanField(default=False, verbose_name=_('Show only for super user'))
    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = BaseModelManager()
    
    def __init__(self, *args, **kwargs):
        super(BaseModel, self).__init__(*args, **kwargs)

    def __str__(self):
        return "%s" % self.id

    class Meta:
        ordering = ['-created_at']
        verbose_name = _('Base Model')
        verbose_name_plural = _('Bases Models')

    def get_api_uri(self):
        return reverse('aparnik-api:models:details', args=[self.id])

    def get_api_review_uri(self):
        return reverse('aparnik-api:reviews:models-list', args=[self.id])

    def get_api_qa_uri(self):
        return reverse('aparnik-api:qa:models-list', args=[self.id])

    def get_api_qa_add_uri(self):
        return reverse('aparnik-api:qa:create', args=[])

    def is_bookmark(self, user):
        if not user.is_authenticated:
            return False
        from aparnik.contrib.bookmarks.models import Bookmark
        order = Bookmark.objects.filter(user_obj=user, model_obj=self)
        if order.count() > 0:
            return True
        return False

    def is_notify(self, user):
        if not user.is_authenticated:
            return False
        from aparnik.contrib.notifiesme.models import NotifyMe
        notifies = NotifyMe.objects.active().filter(user_obj=user, model_obj=self)
        if notifies.count() > 0:
            return True
        return False

    def get_title(self):
        return ''

    def get_description(self):
        return ''

    def get_api_share_uri(self):
        return reverse('aparnik-api:models:share-content', args=[self.id])

    def get_api_notify_me_set_uri(self):
        return reverse('aparnik-api:notifiesme:set', args=[self.id])

    def get_api_bookmark_set_uri(self):
        return reverse('aparnik-api:bookmarks:set', args=[self.id])

    def get_api_review_add_uri(self):
        return reverse('aparnik-api:reviews:create', args=[])

    def has_permit_full_access(self, user):
        if not user.is_authenticated:
            return False
        if user.is_superuser or user.is_admin:
            return True
        return False

    def get_user_for_base_review_notification(self, instance, created):
        return User.objects.none()

    def has_permit_edit(self, user):
        return False

    @property
    def resourcetype(self):
        return self._meta.object_name

    @property
    def review_count(self):
        return self.reviews_set.active().count()

    @property
    def review_count_string(self):
        return human_format(self.review_count)

    @property
    def qa_count(self):
        return self.question_answers_set.active().count()

    @property
    def qa_count_string(self):
        return human_format(self.qa_count)

    @property
    def bookmark_count(self):
        return self.bookmark_obj.active().count()

    @property
    def bookmark_count_string(self):
        return human_format(self.bookmark_count)

    @property
    def visit_count_string(self):
        return human_format(self.visit_count)

    @staticmethod
    def sort_visit(return_key='visit', prefix=''):
        sort = {
            return_key: {
                'label': 'نمایش',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Coalesce(Sum(field_with_prefix('visit_count', prefix)), 0)
                },
                'key_sort': 'sort_count',
            }
        }
        return sort



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Setting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('key', models.CharField(max_length=255, verbose_name='Key')),
                ('value', models.CharField(max_length=255, verbose_name='Value')),
                ('value_type', models.CharField(choices=[('s', 'String'), ('i', 'Int')], max_length=31, verbose_name='Value Type')),
                ('package_name', models.CharField(blank=True, max_length=70, null=True, verbose_name='Package name')),
                ('function_name', models.CharField(blank=True, max_length=70, null=True, verbose_name='Function name')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('update_at', models.DateTimeField(auto_now=True)),
                ('is_show', models.BooleanField(default=True, verbose_name='Is Show')),
            ],
            options={
                'verbose_name': 'setting',
                'verbose_name_plural': 'settings',
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib.auth import get_user_model
from django.contrib.auth.signals import user_logged_in, user_login_failed, user_logged_out
from django.utils.translation import ugettext_lazy as _

from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework import filters
from rest_framework.serializers import as_serializer_error
from rest_framework.validators import ValidationError

from fcm_django.models import FCMDevice

from aparnik.settings import aparnik_settings
from aparnik.utils.utils import convert_iran_phone_number_to_world_number
from aparnik.contrib.users.models import DeviceType, UserToken


UserDetailSerializer = aparnik_settings.USER_DETAIL_SERIALIZER
UserCreateSerializer = aparnik_settings.USER_CREATE_SERIALIZER
UserUpdateSerializer = aparnik_settings.USER_UPDATE_SERIALIZER
UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER
# from slider.api.serializers import SliderDetailsSerializer, Slider
# from notification.models import Notification

User = get_user_model()


def get_user_name(username):
    username = convert_iran_phone_number_to_world_number(username)
    try:
        user = User._default_manager.get_by_natural_key(username)
    except:
        if aparnik_settings.USER_REGISTER_WITH_LOGIN and not aparnik_settings.USER_LOGIN_WITH_PASSWORD:
            user = User(username=username)
            user.save()
        else:
            user = None

    return user


class UserTokenAPIView(APIView):
    permission_classes = [AllowAny]

    """
    A view that returns the count of active users in JSON.
    """
    def post(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}

        try:
            from aparnik.contrib.users.models import DeviceLogin
            from rest_framework_jwt.views import obtain_jwt_token
            from django.http import HttpRequest

            username = request.data.get('username').strip()
            version_number = request.data.get('version_number', None)
            device_id = request.data.get('device_id', None)
            device_type = DeviceType.find(request.data.get('device_type', None))
            os_version = request.data.get('os_version', None)
            device_model = request.data.get('device_model', None)

            user = get_user_name(username=username)

            responce = obtain_jwt_token(request=request._request, format=format)
            if responce.status_code == 200:
                device = None
                if user.is_can_login(device_id=device_id):
                    device = DeviceLogin.objects.create(user=user, version_number=version_number, device_id=device_id,
                                                        os_version=os_version, device_model=device_model,
                                                        device_type=device_type)
                    device.save()

                    if aparnik_settings.IS_FCM:
                        device_fcm = FCMDevice.objects.filter(device_id=device.device_id).order_by('-id').first()
                        if device_fcm:
                            # raise ValidationError({
                            #     'login': _("We can't login. Please Contact with support team.")
                            # })
                            device_fcm.user = user
                            device_fcm.save()

                token = responce.data['token']
                UserToken.objects.create(token=token, user_obj=user, device_obj=device)
            elif responce.status_code >= 400:
                user_login_failed.send(sender=UserTokenAPIView.__class__, request=request, credentials={'username': user.username, 'password': None})

            return responce

        except Exception as e:
            raise ValidationError(as_serializer_error(e))
           

class UserForgetPasswordAPI(APIView):
    permission_classes = [AllowAny]

    """
    A view that returns the count of active users in JSON.
    """
    def post(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}

        try:
            username = request.data.get('username').strip()
            version_number = request.data.get('version_number', None)
            device_id = request.data.get('device_id', None)
            device_type = DeviceType.find(request.data.get('device_type', None))
            os_version = request.data.get('os_version', None)
            device_model = request.data.get('device_model', None)

            user = get_user_name(username=username)
            user.reset_password()

            content['success'] = True
            content['msg'] = ""
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            raise ValidationError({
                'reset_password': _("We can't reset password. Please Contact with support team.")
            })


class UserLogoutAPIView(APIView):
    permission_classes = [IsAuthenticated]

    """
    A view that returns the count of active users in JSON.
    """
    def post(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}

        try:
            user = request.user
            from aparnik.contrib.users.models import DeviceLogin
            version_number = request.data.get('version_number', None)
            device_id = request.data.get('device_id', None)
            device_type = DeviceType.find(request.data.get('device_type', None))
            os_version = request.data.get('os_version', None)
            device_model = request.data.get('device_model', None)
            device_login = DeviceLogin.objects.active().filter(device_id=device_id, user=user).first()
            if device_login:
                device_login.is_active = False
                device_login.save()
                if aparnik_settings.IS_FCM:
                    device = FCMDevice.objects.filter(user=user, device_id=device_login.device_id).first()
                    if device:
                        device.user = None
                        device.save()
            UserToken.objects.filter(user_obj=user, device_obj__device_id=device_login.device_id).update(is_active=False)
            user_logged_out.send(sender=UserLogoutAPIView.__class__, request=request, user=user)
            content['success'] = True
            content['msg'] = ""
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            raise ValidationError({
                'logout': _("We can't logout. Please Contact with support team.")
            })


class UserLoginAPIView(APIView):
    """
    A view that returns the count of active users in JSON.
    """
    def post(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}


        try:
            username = request.data.get('username').strip()
            # user = User._default_manager.get_by_natural_key(username)
            user = get_user_name(username=username)
            version_number = request.data.get('version_number', None)
            device_id = request.data.get('device_id', None)
            device_type = DeviceType.find(request.data.get('device_type', None))
            os_version = request.data.get('os_version', None)
            device_model = request.data.get('device_model', None)
            app_signature = request.data.get('app_signature', None)
        except Exception:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            raise ValidationError({
                'login': _("We can't find this number.")
            })

        try:

            user.OTARequest(app_signature=app_signature)
            # serializer = UserDetailSerializer(user, context={'request': self.request})
            content['data'] = {}
            # content['data']['user'] = serializer.data
            # content['data']['ota'] = user.token
            content['success'] = True
            content['msg'] = ""
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))


class UserSendSMSAPI(APIView):
    """
    A view that returns the count of active users in JSON.
    """
    def post(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}

        try:
            username = request.data.get('username').strip()
            # user = User._default_manager.get_by_natural_key(username)
            user = get_user_name(username=username)
            version_number = request.data.get('version_number', None)
            device_id = request.data.get('device_id', None)
            os_version = request.data.get('os_version', None)
            device_model = request.data.get('device_model', None)
            app_signature = request.data.get('app_signature', None)
        except Exception:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            raise ValidationError({
                'login': _("We can't find this number.")
            })

        try:

            user.OTARequest(app_signature=app_signature)
            # serializer = UserDetailSerializer(user, context={'request': self.request})
            content['data'] = {}
            # content['data']['user'] = serializer.data
            # content['data']['ota'] = user.token
            content['success'] = True
            content['msg'] = ""
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception:
            raise ValidationError({
                'login': _("We can't login. Please Contact with support team.")
            })


class UserVerifySMSAPI(APIView):
    """
    A view that returns the count of active users in JSON.
    """
    def post(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}

        try:
            username = request.data.get('username').strip()
            # user = User._default_manager.get_by_natural_key(username)
            user = get_user_name(username=username)
            version_number = request.data.get('version_number', None)
            password = request.data.get('password', None)
            device_id = request.data.get('device_id', None)
            os_version = request.data.get('os_version', None)
            device_model = request.data.get('device_model', None)

        except:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            raise ValidationError({
                'login': _("We can't find this number.")
            })

        try:

            if user.OTAVerify(password):
                user.verify_mobile = True
                user.save()
            else:
                raise ValidationError
            # serializer = UserDetailSerializer(user, context={'request': self.request})
            content['data'] = {}
            # content['data']['user'] = serializer.data
            # content['data']['ota'] = user.token
            content['success'] = True
            content['msg'] = ""
            status = HTTP_200_OK

        except Exception:
            raise ValidationError({
                'login': _("کد وارد شده اشتباه است.")
            })

        return Response(content, status=status)


class UserListAPIView(ListAPIView):
    serializer_class = UserSummeryListSerializer
    permission_classes = [IsAuthenticated]
    # queryset = User.objects.all()
    filter_backends = [filters.SearchFilter]
    search_fields = ['username', 'username_mention', 'last_name', 'first_name']

    def get_queryset(self):

        return User.objects.active()


class UserSubsetListAPIView(ListAPIView):
    serializer_class = UserSummeryListSerializer
    permission_classes = [IsAuthenticated]
    # queryset = User.objects.all()
    filter_backends = [filters.SearchFilter]
    search_fields = ['username', 'last_name', 'first_name']

    def get_queryset(self):
        user = self.request.user
        return User.objects.filter(pk__in=user.invite_by.get_invited_by().values_list('invite'))


class UserDetailAPIView(RetrieveAPIView):
    serializer_class = UserDetailSerializer
    permission_classes = [IsAuthenticated]
    lookup_url_kwarg = 'username'
    lookup_field = 'username'

    def get_queryset(self):
        user = self.request.user

        return User.objects.filter(username=user.username)


class UserCreateAPIView(CreateAPIView):
    serializer_class = UserCreateSerializer
    queryset = User.objects.all()
    permission_classes = [AllowAny]


#
class UserUpdateAPIView(UpdateAPIView):
    serializer_class = UserUpdateSerializer
    permission_classes = [IsAuthenticated]
    lookup_url_kwarg = 'username'
    lookup_field = 'username'

    def get_object(self, *args, **kwargs):

        user = self.request.user
        return user


class FCMAddTokenAPIView(APIView):
    permission_classes = (AllowAny,)
    """
    A view that returns the count of active users in JSON.
    """
    def post(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}

        try:
            user = None
            if request.user.is_authenticated:
                user = request.user
            device_type = DeviceType.find(request.data.get('device_type', None))
            device_id = request.data.get('device_id', None)
            device_model = request.data.get('device_model', None)
            fcm_token = request.data.get('fcm_token', None)
            if fcm_token is None:
                raise Exception
            fcm = FCMDevice.objects.create(
                registration_id=fcm_token,
                device_id=device_id,
                type=DeviceType.value(device_type),
                user=user
            )
            fcm.save()

            content['success'] = True
            content['msg'] = ""
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            raise ValidationError({
                'fcm': _("We can't add fcm token.")
            })


def jwt_response_payload_handler(token, user=None, request=None):
    if user and request:
        user_logged_in.send(sender=user.__class__, request=request, user=user)
    return {
        'token': token,
        'user': UserDetailSerializer(user, context={'request': request}).data
    }



# Generated by Django 2.2 on 2019-11-03 12:40

from django.db import migrations



def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Page = apps.get_model('pages', 'Page')
    ContentType = apps.get_model('contenttypes', 'ContentType')

    key = ''
    try:
        key = 'home'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='صفحه اصلی',
            english_title=key,
            is_show_in_home=True,
        )

    try:
        key = 'library'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='کتابخانه',
            english_title=key,
            is_show_in_home=False,
        )

    try:
        key = 'category'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='دسته بندی',
            english_title=key,
            is_show_in_home=False,
        )

    try:
        key = 'contact_us'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='تماس با ما',
            english_title=key,
            is_show_in_home=False,
        )

    try:
        key = 'about_us'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='درباره ما',
            english_title=key,
            is_show_in_home=False,
        )

    try:
        key = 'my_products'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='محصولات من',
            english_title=key,
            is_show_in_home=False,
        )

    try:
        key = 'download_manager'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='دانلود منیجر',
            english_title=key,
            is_show_in_home=False,
        )

    try:
        key = 'file_manager'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='مدیریت فایل',
            english_title=key,
            is_show_in_home=False,
        )

    try:
        key = 'user_invitation'
        Page.objects.get(english_title=key)
    except Exception:
        Page.objects.create(
            title='دعوت از دوستان',
            english_title=key,
            is_show_in_home=False,
        )

    new_ct = ContentType.objects.get_for_model(Page)
    Page.objects.filter(polymorphic_ctype__isnull=True).update(polymorphic_ctype=new_ct)

def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Page = apps.get_model('pages', 'Page')

    keys = [
        'home',
        'library',
        'category',
        'contact_us',
        'about_us',
        'my_products',
        'download_manager',
        'file_manager',
        'user_invitation',
    ]

    for key in keys:
        try:
            Page.objects.get(english_title=key).delete()
        except Exception:
            pass



class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0004_auto_20190114_1537'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-25 22:27


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contactus', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contactus',
            name='content',
            field=models.TextField(blank=True, null=True, verbose_name='Content'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.http import Http404

from aparnik.utils.utils import is_app_installed

def send_to_bank(request, payment):
    try:
        if not is_app_installed('aparnik.packages.bankgateways.bmi'):
            raise Exception()
        from aparnik.packages.bankgateways.bmi.views import send_request as bmi_send_request
        return bmi_send_request(request=request, payment=payment)
    except:
        pass
    try:
        if not is_app_installed('aparnik.packages.bankgateways.zarinpals'):
            raise Exception()
        from aparnik.packages.bankgateways.zarinpals.views import send_request as zarinpal_send_request
        return zarinpal_send_request(request=request, payment=payment)
    except:
        pass

    raise Http404


from django.db.models import Sum, Q
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from collections import OrderedDict

from rest_framework.views import APIView
from rest_framework.permissions import IsAdminUser, AllowAny
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.exceptions import ValidationError
from rest_framework.serializers import as_serializer_error
from rest_framework.generics import ListAPIView
from rest_framework.response import Response
from rest_framework import pagination


class BaseSortAPIView(APIView):
    permission_classes = [AllowAny]
    command_dict = {}

    def __init__(self, *args, **kwargs):
        super(BaseSortAPIView, self).__init__(*args, **kwargs)
        self.command_dict = {

        }

    def get(self, request, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}
        try:
            content = self.get_sort_content_list()
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))

    def get_sort_content_list(self):
        return [{'key': key, 'label': value['label']} for key, value in list(self.command_dict.items())]

    def isAllowKey(self, key):
        return key in self.command_dict

    def get_query_sort(self, queryset):

        ordering = None

        if 'ordering' in self.request.query_params:
            ordering = self.request.query_params['ordering']
        else:
            return queryset

        if not self.isAllowKey(ordering.lstrip('-')):
            raise ValidationError({'ordering': [_('This is not valid order parameters for sorting.')]})

        command = self.command_dict[ordering.lstrip('-')]

        if ordering[0] == '-':
            command['key_sort'] = '-' + command['key_sort']

        return queryset.filter(command['queryset_filter']).annotate(**command['annotate_command']).order_by(
            command['key_sort']).distinct()


class BaseListAPIView(ListAPIView):
    # paginator = AdminModelPaginator()
    # permission_classes = [IsAdminUser]

    def filter_queryset(self, queryset, *args, **kwargs):
        queryset = super(BaseListAPIView, self).filter_queryset(queryset)
        sort_api = self.get_sort_api(self.request)
        queryset = sort_api.get_query_sort(queryset)
        return queryset


    def get_sort_api(self, request):
        return BaseSortAPIView(request=request)



class AdminModelPaginator(pagination.PageNumberPagination):

    def get_paginated_response(self, data):
        total_count = 0
        try:
            total_count = self.page.paginator.object_list.aggregate(Sum('sort_count'))['sort_count__sum']
        except:
            total_count = 0

        return Response(OrderedDict([
            ('count', self.page.paginator.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('total_count', total_count),
            ('results', data)
        ]))


class ModelAdminSortAPIView(BaseSortAPIView):
    permission_classes = [IsAdminUser]

    def __init__(self, *args, **kwargs):
        super(ModelAdminSortAPIView, self).__init__(*args, **kwargs)
        command_dict = {

        }

        self.command_dict.update(command_dict)


class AdminListAPIView(ListAPIView):
    paginator = AdminModelPaginator()
    permission_classes = [IsAdminUser]

    def filter_queryset(self, queryset, *args, **kwargs):
        queryset = super(AdminListAPIView, self).filter_queryset(queryset)
        sort_api = self.get_sort_api(self.request)
        queryset = sort_api.get_query_sort(queryset)
        return queryset


    def get_sort_api(self, request):
        return ModelAdminSortAPIView(request=self.request)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-06-12 10:35


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0010_basemodel_visit_count'),
    ]

    operations = [
        migrations.AlterField(
            model_name='basemodel',
            name='review_average',
            field=models.FloatField(default=0.0, verbose_name='\\u0645\\u06cc\\u0627\\u0646\\u06af\\u06cc\\u0646 \\u0646\\u0638\\u0631\\u0627\\u062a'),
        ),
    ]



"""
Context Processor

To include the notifications varialbe in all templates.
"""


def notifications(request):
    """Make notifications available in all templates."""
    if not request.user.is_authenticated:
        return {}

    return {'notifications': request.user.notifications}



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-27 15:36


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0002_auto_20181026_1745'),
    ]

    operations = [
        migrations.AddField(
            model_name='basemodel',
            name='update_needed',
            field=models.BooleanField(default=False, verbose_name='Update needed'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 13:01


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='orderitem',
            managers=[
            ],
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.users.admin import get_update_at
from aparnik.contrib.basemodels.admin import BaseModelAdmin
from .models import Slider, SliderImage, SliderLink, SliderVideo, SliderBaseModel, SliderSocialNetwork


# Register your models here.
class SliderAdmin(BaseModelAdmin):
    fields = ['name', 'image', 'is_active']
    exclude = []
    dynamic_raw_id_fields = []
    # list_display = ['id', 'name', 'is_active', get_update_at, ]
    list_display = ['name', get_update_at, ]
    search_fields = ['name', ]
    # list_filter = ['is_active', 'is_home', 'is_library', 'created_at', ]
    list_filter = ['is_active', 'created_at', ]
    raw_id_fields = ['image']

    def __init__(self, *args, **kwargs):
        Klass = SliderAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Slider


class SliderImageAdmin(SliderAdmin):
    fields = []
    exclude = []
    dynamic_raw_id_fields = []
    list_display = []
    search_fields = []
    list_filter = []
    raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = SliderImageAdmin
        Klass_parent = SliderAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = SliderImage


class SliderSocialNetworkAdmin(SliderAdmin):
    fields = ['social']
    list_display = []
    list_filter = []
    raw_id_fields = ['social']
    search_fields = ['social__title', 'social__link']
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = SliderSocialNetworkAdmin
        Klass_parent = SliderAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = SliderSocialNetwork


class SliderLinkAdmin(SliderAdmin):
    fields = ['link']
    list_display = []
    list_filter = []
    raw_id_fields = []
    search_fields = []
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = SliderLinkAdmin
        Klass_parent = SliderAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = SliderLink


class SliderVideoAdmin(SliderAdmin):
    fields = ['video']
    list_display = []
    list_filter = []
    raw_id_fields = ['video']
    search_fields = []
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = SliderVideoAdmin
        Klass_parent = SliderAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = SliderVideo


class SliderBaseModelAdmin(SliderAdmin):
    fields = ['model']
    list_display = []
    list_filter = []
    raw_id_fields = ['model']
    search_fields = []
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = SliderBaseModelAdmin
        Klass_parent = SliderAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = SliderBaseModel

    def get_form(self, request, obj=None, **kwargs):
        form = super(SliderBaseModelAdmin, self).get_form(request, obj, **kwargs)
        if is_app_installed('aparnik.packages.shops.products'):

            from aparnik.packages.shops.products.models import Product
            form.base_fields['model'].queryset = Product.objects.active()

        return form


if is_app_installed('aparnik.contrib.segments'):

    from aparnik.contrib.segments.admin import SegmentAdmin
    from .models import SliderSegment


    class SliderSegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        exclude = []
        raw_id_fields = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = SliderSegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = SliderSegment


    admin.site.register(SliderSegment, SliderSegmentAdmin)

admin.site.register(Slider, SliderAdmin)
admin.site.register(SliderImage, SliderImageAdmin)
admin.site.register(SliderSocialNetwork, SliderSocialNetworkAdmin)
admin.site.register(SliderLink, SliderLinkAdmin)
admin.site.register(SliderVideo, SliderVideoAdmin)
admin.site.register(SliderBaseModel, SliderBaseModelAdmin)



# Generated by Django 2.2 on 2019-07-14 16:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0012_auto_20190701_0911'),
    ]

    operations = [
        migrations.AddField(
            model_name='filefield',
            name='file_external_url',
            field=models.URLField(blank=True, null=True, verbose_name='url'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='type',
            field=models.CharField(blank=True, choices=[('M', 'فیلم'), ('V', 'صدا'), ('P', 'پی دی اف'), ('I', 'عکس'), ('L', 'لینک')], max_length=1, null=True, verbose_name='نوع فایل'),
        ),
    ]



from django.conf.urls import url, include

from .views import TicketConversationListAPIView, TicketConversationCreateAPIView

app_name = 'tickets'

urlpatterns = [
    url(r'^$', TicketConversationListAPIView.as_view(), name='list'),
    url(r'^create/$', TicketConversationCreateAPIView.as_view(), name='create'),

]


"""Tests."""

from django.test import TestCase
from django.contrib.auth import get_user_model

from .. import MessagingError
from ..models import Messaging
from ..signals import read, notify


class GeneralTestCase(TestCase):
    """Tests for General functionality."""

    User = get_user_model()

    @classmethod
    def setUpTestData(cls):
        """Create Users."""
        cls.user1 = cls.User.objects.create_user(
            username='user1@gmail.com', password='password'
        )

        cls.user2 = cls.User.objects.create(
            username='user2@gmail.com', password='password'
        )

    def test_to_json_without_extra_data(self):
        """
        If the extra_data argument is ommitted,

        the default should be an empty dictionary
        """
        # Create notification
        notification = Messaging.objects.create(
            source=self.user2, source_display_name='User 2',
            recipient=self.user1, action='Notified',
            category='General notification', obj=1, url='http://example.com',
            short_description='Short Description', is_read=False,
        )

        self.assertEqual(
            notification.to_json(),
            {
                'source': self.user2.id, 'source_display_name': 'User 2',
                'recipient': self.user1.id, 'action': 'Notified',
                'category': 'General notification', 'obj': 1,
                'short_description': 'Short Description',
                'extra_data': {}, 'channels': '',
                'url': 'http://example.com', 'is_read': False
            }
        )

    def test_to_json_with_extra_data(self):
        """Test to_json method with extra data."""
        notification = Messaging.objects.create(
            source=self.user2, source_display_name='User 2',
            recipient=self.user1, action='Notified',
            category='General notification', obj=1, url='http://example.com',
            short_description='Short Description', is_read=False,
            extra_data={'hello': 'world'}
        )

        self.assertEqual(
            notification.to_json(),
            {
                'source': self.user2.id, 'source_display_name': 'User 2',
                'recipient': self.user1.id, 'action': 'Notified',
                'category': 'General notification', 'obj': 1,
                'short_description': 'Short Description', 'channels': '',
                'url': 'http://example.com', 'extra_data': {'hello': 'world'},
                'is_read': False,
            }
        )


class NotificationSignalTestCase(TestCase):
    """Tests for the notification signals."""

    User = get_user_model()

    @classmethod
    def setUpTestData(cls):
        """Create Users."""
        cls.user1 = cls.User.objects.create_user(
            username='user1@gmail.com', password='password'
        )

        cls.user2 = cls.User.objects.create(
            username='user2@gmail.com', password='password'
        )

    def test_user_cant_read_others_notifications(self):
        """A user should only be able to read THEIR notifications."""
        # Create Notification for User2
        notification = Messaging.objects.create(
            source=self.user1, source_display_name='User 1',
            recipient=self.user2, action='Notified',
            category='General notification', obj=1, url='http://example.com',
            is_read=False
        )

        # Try and Read the notification as User1
        self.assertRaises(
            MessagingError,

            read.send,
            sender=self.__class__, notify_id=notification.id,
            recipient=self.user1
        )

    def test_user_can_read_notifications(self):
        """A user can read their notification"""
        # Create Notification for User1
        notification = Messaging.objects.create(
            source=self.user2, source_display_name='User 2',
            recipient=self.user1, action='Notified',
            category='General notification', obj=1, url='http://example.com',
            is_read=False
        )

        # Try and Read the notification as user1
        read.send(
            sender=self.__class__, notify_id=notification.id,
            recipient=self.user1
        )

        notification.refresh_from_db()
        self.assertEqual(notification.is_read, True)

    def test_silent_notification(self):
        """Test Silent notifications."""
        notify.send(
            sender=self.__class__, source=self.user2,
            source_display_name='User 2', recipient=self.user1,
            action='Notified', category='Silent notification', obj=1,
            url='http://example.com',
            short_description='Short Description', is_read=False,
            silent=True, channels=('console',)
        )

        notifications = Messaging.objects.all()

        self.assertEqual(tuple(notifications), tuple())


class JSONFieldTestCase(TestCase):
    """Test the Custom JSONField."""

    User = get_user_model()

    @classmethod
    def setUpTestData(cls):
        """Create Users."""
        cls.user1 = cls.User.objects.create_user(
            username='user1@gmail.com', password='password'
        )

        cls.user2 = cls.User.objects.create(
            username='user2@gmail.com', password='password'
        )

    def test_raise_exception(self):
        """
        Should raise an exception

        When we try to save objects that can be serialized by the json module.
        """
        kwargs = {
            'sender': self.__class__, 'source': self.user2,
            'source_display_name': 'User 2', 'recipient': self.user1,
            'action': 'Notified', 'category': 'General notification',
            'obj': 1, 'short_description': 'Short Description',
            'url': 'http://example.com', 'is_read': False,
            'extra_data': {'hello': lambda x: 'world'},
            'channels': ('console',)
        }

        self.assertRaises(TypeError, notify.send, **kwargs)

    def test_json_decode(self):
        """Should return a dictionary back."""
        notify.send(
            sender=self.__class__, source=self.user2,
            source_display_name='User 2', recipient=self.user1,
            action='Notified', category='Notification with extra data', obj=1,
            url='http://example.com',
            short_description='Short Description', is_read=False,
            extra_data={'hello': 'world'}, channels=('console',)
        )

        notification = Messaging.objects.last()

        self.assertEqual(
            notification.extra_data, {'hello': 'world'}
        )


class TestListField(TestCase):
    """Tests for the list field."""

    User = get_user_model()

    @classmethod
    def setUpTestData(cls):
        """Create Users."""
        cls.user1 = cls.User.objects.create_user(
            username='user1@gmail.com', password='password'
        )

        cls.user2 = cls.User.objects.create(
            username='user2@gmail.com', password='password'
        )

    def test_should_return_list(self):
        """Should return a list of channels back."""
        notify.send(
            sender=self.__class__, source=self.user2,
            source_display_name='User 2', recipient=self.user1,
            action='Notified', category='Notification with extra data', obj=1,
            url='http://example.com',
            short_description='Short Description', is_read=False,
            extra_data={'hello': 'world'}, channels=('console', 'console')
        )

        notification = Messaging.objects.last()

        self.assertEqual(
            notification.to_json()['channels'], ['console', 'console']
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-14 15:24


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0007_auto_20181114_1230'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='further_details',
            field=models.TextField(blank=True, null=True, verbose_name='Extra Description'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from aparnik.contrib.users.models import PhoneField
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.socials.models import SocialNetwork
from aparnik.contrib.shortblogs.models import ShortBlog
from aparnik.contrib.filefields.models import FileField
from aparnik.contrib.sliders.models import SliderSegment


# Create your models here.
class InformationManager(models.Manager):
    def get_queryset(self):
        return super(InformationManager, self).get_queryset()

    def get_active(self):
        return self.get_queryset().filter(is_active=True)


class Information(models.Model):
    website = models.URLField(
        max_length=255,
        verbose_name=_('Website')
    )

    address = models.CharField(
        max_length=255,
        verbose_name=_('Address')
    )

    slider_segment_obj = models.ForeignKey(SliderSegment, on_delete=models.CASCADE,
                                           null=True, blank=True, related_name='aboutus_sliders',
                                           verbose_name=_('Slider Segment'))
    phone = PhoneField(
        mobile=False,
        verbose_name=_('Phone Number')
    )

    email = models.EmailField(
        null=True,
        blank=True,
        verbose_name=_('Email')
    )

    about_us = models.TextField(
        verbose_name=_('About Us'),
    )

    image = models.ForeignKey(FileField, on_delete=models.CASCADE, verbose_name=_('Image'))

    is_active = models.BooleanField(
        verbose_name=_('Is Active'),
        default=True
    )
    socials = models.ManyToManyField(SocialNetwork, verbose_name=_('Socials'))
    short_blogs = models.ManyToManyField(ShortBlog, blank=True, verbose_name=_('Content Segment'))
    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = InformationManager()

    def save(self, *args, **kwargs):

        return super(Information, self).save(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        super(Information, self).__init__(*args, **kwargs)

    class Meta:
        # ordering = ['latitude', 'longitude']
        verbose_name = _('Information')
        verbose_name_plural = _('Informations')



# -*- coding: utf-8 -*-


from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.db.models.signals import post_save
from django.urls import reverse

from django_enumfield import enum

from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.packages.shops.products.models import Product


User = get_user_model()


# Create your models here.
class NotificationType(enum.Enum):
    SYSTEM = 0
    SINGLE_USER = 1
    ALL_USER = 2

    labels = {
        SYSTEM: _('System'),
        SINGLE_USER: _('Single User'),
        ALL_USER: _('All User')
    }


class NotificationManager(BaseModelManager):

    def get_queryset(self):
        return super(NotificationManager, self).get_queryset()

    def notification_type(self, type):
        return self.get_queryset().filter(notification_send_type=type)

    def get_this_user(self, user):
        if not user.is_authenticated:
            return Notification.objects.none()
        return self.get_queryset().filter(membership__user=user).distinct().order_by('-created_at')

    def unread(self, user):
        # Remember (membership__user=user, membership__is_read=False) should must be in one prantisis.
        if not user.is_authenticated:
            return Notification.objects.none()
        return self.active().filter(membership__user=user, membership__is_read=False).order_by('-created_at')

    def unread_count(self, user):
        if not user.is_authenticated:
            return Notification.objects.none()

        return self.unread(user=user).count()

    def read(self, user):
        if not user.is_authenticated:
            return Notification.objects.none()

        read = self.unread(user=user)
        for r in read:
            r.read(user=user)
        return read.count()

    def send_notification(self, users, type, title, description, from_user_obj=None, model_obj=None):
        if users is None:
            return None
        notification = Notification.objects.create(
            type=type,
            title=title,
            description=description,
            update_needed=True,
            from_user_obj=from_user_obj,
            model_obj=model_obj,
            notification_send_type=NotificationType.SYSTEM,
        )
        memberships = []

        for user in users:
            memberships.append(Membership(user=user, notification=notification))

        Membership.objects.bulk_create(memberships)

        return notification


class Notification(BaseModel):
    NOTIFICATION_INFO = 'i'
    NOTIFICATION_CHAT = 'ch'
    NOTIFICATION_SUCCESS = 's'
    NOTIFICAITON_WARNING = 'w'
    NOTIFICAITON_ERROR = 'e'

    NOTIFICATION_TYPE = (
        (NOTIFICATION_INFO, 'اطلاعات'),
        (NOTIFICATION_CHAT, _('Chat')),
        (NOTIFICATION_SUCCESS, 'موفقیت'),
        (NOTIFICAITON_WARNING, 'هشدار'),
        (NOTIFICAITON_ERROR, 'خطا')
    )
    from_user_obj = models.ForeignKey(User, related_name='notificaiton_from_user', null=True, blank=True, on_delete=models.CASCADE, verbose_name='از کاربر')
    users = models.ManyToManyField(User, through='Membership', verbose_name=_('Membership'))
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    description = models.TextField(verbose_name=_('Message'))
    type = models.CharField(max_length=2, choices=NOTIFICATION_TYPE, verbose_name=_('Type'))
    notification_send_type = enum.EnumField(NotificationType, default=NotificationType.ALL_USER, verbose_name=_('Notification send type'))
    model_obj = models.ForeignKey(BaseModel, related_name='notification_model_obj', null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('Model'))
    datetime = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Datetime'))
    products_buy = models.ManyToManyField(Product, blank=True, related_name='notification_products_buy', verbose_name=_('Purchase specific products'))
    products_buy_did_not = models.ManyToManyField(Product, blank=True, related_name='notification_products_buy_did_not', verbose_name=_('Did not purchase specific products '))
    description_for_admin = models.TextField(blank=True, null=True, verbose_name=_('Description for admin'))
    sent_result = models.TextField(blank=True, null=True, verbose_name=_('Sent result'))

    objects = NotificationManager()

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = _('Notification')
        verbose_name_plural = _('Notifications')

    def save(self, *args, **kwargs):
        if not self.id:
            self.update_needed = True
        return super(Notification, self).save(*args, **kwargs)

    def get_notification_type_json_display(self):

        icon = ""

        if self.type == self.NOTIFICATION_INFO:

            icon = "http://www.freeiconspng.com/uploads/info-icon-7.png"
        elif self.type == self.NOTIFICATION_SUCCESS:

            icon = "http://www.freeiconspng.com/uploads/success-icon-10.png"
        elif self.type == self.NOTIFICAITON_WARNING:

            icon = "http://www.freeiconspng.com/uploads/black-warning-icon-20.png"
        elif self.type == self.NOTIFICAITON_ERROR:

            icon = "http://www.freeiconspng.com/uploads/error-icon-4.png"
        json = {
            'icon': icon,
            'name': self.get_type_display()
        }
        return json

    def get_user(self):
        return self.users.first()

    def get_user_is_read(self, user):
        return self.membership_set.filter(user=user).first().is_read

    def read(self, user):
        return self.membership_set.filter(user=user, is_read=False).update(is_read=True)

    def get_api_read_uri(self):
        return reverse('aparnik-api:notifications:read', args=[self.id])


class Membership(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    notification = models.ForeignKey(Notification, on_delete=models.CASCADE, verbose_name=_('Notification'))
    is_read = models.BooleanField(default=False, verbose_name=_('Is Read'))

    class Meta:
        verbose_name = _('Membership')
        verbose_name_plural = _('Memberships')


class NotificaitonForSingleUserManager(NotificationManager):
    pass


class NotificationForSingleUser(Notification):
    objects = NotificaitonForSingleUserManager()

    class Meta:
        proxy = True
        verbose_name = _('Notification For Single User')
        verbose_name_plural = _('Notifications For Single User')

    def save(self, *args, **kwargs):
        self.full_clean()
        self.notification_send_type = NotificationType.SINGLE_USER
        return super(NotificationForSingleUser, self).save(*args, **kwargs)


class NotificaitonSystemManager(NotificationManager):
    pass


class NotificationSystem(Notification):

    objects = NotificaitonSystemManager()

    class Meta:
        proxy = True
        verbose_name = _('Notification System')
        verbose_name_plural = _('Notifications System')

    def save(self, *args, **kwargs):
        self.full_clean()
        self.notification_send_type = NotificationType.SYSTEM
        return super(NotificationSystem, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext as _
from django.core.validators import ValidationError

from s3direct.fields import S3DirectField
from aparnik.utils.utils import get_request
from aparnik.settings import aparnik_settings, Setting
from aparnik.utils.utils import document_directory_path
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager

import magic
import humanize

request = get_request()


class FileFieldManager(BaseModelManager):

    def get_queryset(self):
        return super(FileFieldManager, self).get_queryset()

    def active(self, user=None):
        return super(FileFieldManager, self).active(user).filter(is_lock=False)

    def encrypt_needed(self):
        return self.get_queryset().filter(is_lock=True, multi_quality_processing=2)

    def multi_quality(self):
        return self.get_queryset().filter(is_lock=True, multi_quality_processing=0)


class FileField(BaseModel):
    FILE_MOVIE = 'M'
    FILE_VOICE = 'V'
    FILE_PDF = 'P'
    FILE_IMAGE = 'I'
    FILE_LINK = 'L'
    FILE_TYPE = (
        (FILE_MOVIE, _('Movie')),
        (FILE_VOICE, _('Voice')),
        (FILE_PDF, _('PDF')),
        (FILE_IMAGE, _('Image')),
        (FILE_LINK, _('Link')),
    )
    title = models.CharField(max_length=100, blank=True, null=True, verbose_name=_('Title'))
    file_s3 = S3DirectField(dest='file', blank=True, null=True, verbose_name=_('File'))
    file_direct = models.FileField(upload_to=document_directory_path, blank=True, null=True, verbose_name=_('File'))
    file_external_url = models.URLField(null=True, blank=True, verbose_name=_('url'))
    size = models.IntegerField(default=0, verbose_name=_('File Size'))
    type = models.CharField(max_length=1, null=True, blank=True, choices=FILE_TYPE, verbose_name=_('File Type'))
    is_lock = models.BooleanField(default=False, verbose_name=_('Is lock?'))
    is_encrypt_needed = models.BooleanField(default=False, verbose_name=_('Is encrypt needed?'))
    # این فیلد فقط در کد ها استفاده می شود . در کلین نمی توانیم این کار رو انجام بدیم بدلیل اینکه دو بار فراخوانی می شود.
    is_decrypt_needed = models.BooleanField(default=False, verbose_name=_('Is decrypt needed?'))
    password = models.CharField(max_length=64, default='', blank=True, null=True, verbose_name=_('Password'))
    iv = models.CharField(max_length=64, default='', blank=True, null=True, verbose_name=_('IV'))
    seconds = models.BigIntegerField(default=0, verbose_name=_('Time'))
    multi_quality = models.BooleanField(default=False, verbose_name=_('Multi Quality'))
    # -1 doesnt set, 0 not begin, 1 processing, 2 processed
    multi_quality_processing = models.IntegerField(default=-1, verbose_name=_('Multi Quality Processing'))

    objects = FileFieldManager()

    class Meta:
        verbose_name = _('File')
        verbose_name_plural = _('Files')

    def __str__(self):
        return "%s - %s" % (self.id, self.title)

    class Meta:
        verbose_name = _('File Field')
        verbose_name_plural = _('Files Field')
        app_label = 'filefields'

    def clean(self):
        # Don't allow complete action entries.
        if self.id:
            obj = FileField.objects.get(pk=self.id)
        else:
            obj = self

        if self.id and obj.is_lock and self.is_lock:
            raise ValidationError(_('This record locked.'))

        if not self.file_direct and not self.file_s3 and not self.file_external_url:
            raise ValidationError(_('File cannot be blank.'))

        if not aparnik_settings.AWS_ACTIVE and not self.file_external_url:

            if not self.id or self.file_direct.path != obj.file_direct.path and self.is_encrypt_needed:
                # encrypt
                self.is_lock = True
            elif not self.is_encrypt_needed and obj.is_encrypt_needed:
                self.is_decrypt_needed = True
                self.is_lock = True
            elif self.is_encrypt_needed and not obj.is_encrypt_needed:
                self.is_lock = True
        # if not self.type:
        #     raise ValidationError({
        #         'type': [_('Value cannot be blank.')],
        #     })

        if self.file_direct._file:
            path = self.file_direct._file.temporary_file_path()
            f = magic.Magic(mime=True, uncompress=False)
            mime = f.from_file(path)
            self.type = self.detect_mime_type(mime)

            if not self.type:
                f = magic.Magic()
                mime = f.from_file(path)
                self.type = self.detect_by_readable_description(mime)

        if self.file_external_url:
            self.type = self.FILE_LINK

        if not self.type:
            raise ValidationError(_('type %s doesnt support.' % mime))

        if self.type == FileField.FILE_MOVIE and self.multi_quality and self.multi_quality_processing == -1:
            # add to queue
            self.multi_quality_processing = 0
            self.is_lock=True
        elif self.multi_quality_processing == -1:
            self.multi_quality_processing = 2

        return super(FileField, self).clean()

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(FileField, self).save(*args, **kwargs)

    def url(self, request=request, max_quality=True):

        if self.type == self.FILE_LINK:
            return self.file_external_url

        file = self.file_s3 if aparnik_settings.AWS_ACTIVE else self.file_direct
        # For check value if empty change and check the another one
        if not file:
            file = self.file_s3 if not aparnik_settings.AWS_ACTIVE else self.file_direct

        if hasattr(file, 'url'):
            url = request.build_absolute_uri(file.url)
            if not max_quality:
                url = '%s%s'%(url, self.file_another_quality)
            return url

        return file if file else None

    @property
    def file_size(self):
        file = self.file_s3 if aparnik_settings.AWS_ACTIVE else self.file_direct

        if not file:
            file = self.file_s3 if not aparnik_settings.AWS_ACTIVE else self.file_direct
        try:
            if hasattr(file, 'size'):
                return file.size
        except:
            pass

        return self.size

    @property
    def file_another_quality(self):
        return 'anotherquality.mp4'

    @property
    def file_size_readable(self):
        return humanize.naturalsize(self.file_size, gnu=True)

    @staticmethod
    def type_icon_url_with_custom_type(type=FILE_IMAGE):
        return Setting.objects.get(key='FILE_TYPE_%s_ICON' % type).get_value()

    @property
    def type_icon_url(self):
        return Setting.objects.get(key='FILE_TYPE_%s_ICON' % self.type).get_value()

    def detect_mime_type(self, mime):

        mime = mime.lower()
        types_map = {
            self.FILE_IMAGE: [
                'image/png',
                'image/jpeg',
                'image/jpg',
            ],
            self.FILE_MOVIE: [
                'video/mpeg',
                'video/mp4',
                'video/x-matroska',
            ],
            self.FILE_PDF: [
                'application/pdf'
            ],
            self.FILE_VOICE: [
                'audio/mpeg',
            ]
        }

        for key in types_map:
            for value in types_map[key]:
                if mime == value:
                    return key
        return None

    def detect_by_readable_description(self, readable_description):

        readable_description = readable_description.lower()

        if readable_description.find('jpeg') != -1 or readable_description.find('png') != -1:

            return self.FILE_IMAGE
        elif readable_description.find('mpeg') != -1:

            return self.FILE_MOVIE
        elif readable_description.find('audio') != -1:

            return self.FILE_VOICE
        elif readable_description.find('pdf') != -1:

            return self.FILE_PDF

        return None



# -*- coding: utf-8 -*-


from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer
from ..models import Category


# Category List Serializer
class CategoryListSerializer(BaseModelListSerializer):
    url_model = serializers.SerializerMethodField()
    image = serializers.SerializerMethodField()
    child_count = serializers.SerializerMethodField()
    url_childs = serializers.SerializerMethodField()

    class Meta:
        model = Category
        fields = BaseModelDetailSerializer.Meta.fields + [
            'title',
            'image',
            'url_model',
            'child_count',
            'url_childs'
        ]

    def get_url_model(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_model_list_uri())

    def get_image(self, obj):
        from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
        return FileFieldListSerailizer(obj.image, many=False, read_only=True, context=self.context).data

    def get_child_count(self, obj):
        return Category.objects.childs_count(category_obj=obj)

    def get_url_childs(self, obj):
        if Category.objects.childs_count(obj) > 0:
            return self.context['request'].build_absolute_uri(obj.get_api_child_uri())
        return None


# Category Details Serializer
class CategoryDetailsSerializer(CategoryListSerializer, BaseModelDetailSerializer):
    parent = serializers.SerializerMethodField()

    class Meta:
        model = Category
        fields = CategoryListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [
            'description',
            'parent',
        ]

    def get_parent(self, obj):
        return CategoryListPolymorphicSerializer(obj.parent_obj, many=False, read_only=True,
                                                 context=self.context).data if obj.parent_obj is not None else None


# Category List Polymorphic
class CategoryListPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            Category: CategoryDetailsSerializer,
        }
        super(CategoryListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Category Details Polymorphic
class CategoryDetailsPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            Category: CategoryDetailsSerializer,
        }
        super(CategoryDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.api.serializers import BaseSegmentListSerializer, BaseSegmentDetailSerializer
    from ..models import CategorySegment

    # Base Segment ListSegmentReview
    class SegmentCategoryListSerializer(BaseSegmentListSerializer):
        class Meta:
            model = CategorySegment
            fields = BaseSegmentListSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


    # Book Segment Details
    class SegmentCategoryDetailSerializer(BaseSegmentDetailSerializer, SegmentCategoryListSerializer):
        class Meta:
            model = CategorySegment
            fields = SegmentCategoryListSerializer.Meta.fields + BaseSegmentDetailSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentDetailSerializer.Meta.read_only_fields + SegmentCategoryListSerializer.Meta.read_only_fields + []



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-04-13 12:32


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('invitation', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='invite',
            options={'verbose_name': 'Invite', 'verbose_name_plural': 'Invites'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-28 13:50


from django.db import migrations


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    key = ''
    try:
        key = 'MANAGER_PRODUCT_ID'
        Setting.objects.get(key=key)
    except Exception:
        Product = apps.get_model('products', 'Product')
        ContentType = apps.get_model('contenttypes', 'ContentType')

        product = Product.objects.create(
            price_fabric=1,
            title='محصول مدیریتی',
            is_show_only_for_super_user=True,
            aparnik_bon_return_value=1,
            maximum_use_aparnik_bon_value=1,
            aparnik_bon_return_expire_value=0,
        )

        new_ct = ContentType.objects.get_for_model(Product)
        Product.objects.filter(polymorphic_ctype__isnull=True).update(polymorphic_ctype=new_ct)

        Setting.objects.create(
            title='آی دی محصول مدیریتی',
            key=key,
            value=str(product.id),
            value_type='i',
            is_show=False,
            is_variable_in_home=False,
        )


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0013_auto_20190123_1447'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 2.2 on 2019-06-26 15:49

from django.db import migrations



def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Review = apps.get_model('reviews', 'Review')
    for obj in Review.objects.all():
        obj.user_obj_2 = obj.user_obj
        obj.save()


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    # FileField = apps.get_model('filefields', 'FileField')
    Review = apps.get_model('reviews', 'Review')
    for obj in Review.objects.all():
        obj.user_obj = obj.user_obj_2
        obj.save()



class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0009_review_user_obj_2'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



"""Frontend urls."""

from django.conf.urls import url

from . import views


app_name = 'messaging'

urlpatterns = [
    url(r'^$', views.NotificationsView.as_view(),
        name='notifications_view'),
    url(r'^generate-notification/$', views.GenerateNotification.as_view())
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 16:17


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filefield',
            name='size',
            field=models.IntegerField(default=0, verbose_name='File Size'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-09 13:21


from django.db import migrations


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')

    key = ''
    try:
        key = 'ORDER_CODE_PREFIX'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='پیشوند کد فاکتور',
            key=key,
            value='APARNIK',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )
    try:
        key = 'POSTAL_COST'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='هزینه پستی',
            key=key,
            value=0,
            value_type='i',
            is_show=False,
            is_variable_in_home=False,
        )


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0007_auto_20190105_1314'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
import requests
from django.shortcuts import render
from django.urls import reverse
from django.http import HttpResponse
from django.shortcuts import redirect, get_object_or_404
from django.http import Http404
from zeep import Client
import datetime
import json
from Crypto.Cipher import DES3
import base64
import logging

from aparnik.packages.shops.payments.models import Payment
from aparnik.packages.shops.orders.models import Order
from aparnik.settings import aparnik_settings, Setting
from aparnik.packages.bankgateways.zarinpals.models import Bank
# from .models import Bank

logging.basicConfig()

def send_request(request, payment):
    order = payment.order_obj
    order_id = order.id
    total_cost = int(order.get_total_cost() * 10)
    # TODO: change the link
    API_URL_REQUEST = 'https://sadad.shaparak.ir/vpg/api/v0/Request/PaymentRequest'
    call_back_url = request.build_absolute_uri(reverse('aparnik:bank_gateways:bmi:verify', args=[payment.uuid]))
    merchent_code = Setting.objects.get(key='BANK_MELLI_MERCHENT_CODE').get_value()
    terminal_id = Setting.objects.get(key='BANK_MELLI_TERMINAL_CODE').get_value()
    time_now = datetime.datetime.now().strftime('%m/%d/%Y %H:%M:%S %P')

    init_data = {
        'TerminalId': terminal_id,
        'MerchantId': merchent_code,
        'Amount': total_cost,
        'SignData': encrypt_des3('%s;%s;%s' % (terminal_id, order_id, total_cost)),
        'ReturnUrl': call_back_url,
        'LocalDateTime': time_now,
        'OrderId': order_id,
        'AdditionalData':  'oi:%s-ou:%s'%(order_id, order.user.username),
    }

    init_response = requests.post(API_URL_REQUEST, json=init_data)
    response_json = get_json(init_response)
    if response_json['ResCode'] == '0':
        token = response_json['Token']
        url = "https://sadad.shaparak.ir/VPG/Purchase?Token=%s"%token

        bank = Bank.objects.create(authority_id=response_json['Token'],
                                   status=0,
                                   payment=payment)

        return redirect(url)
    else:
        logging.log(logging.INFO, response_json)
        print(response_json)
        raise Http404  # HttpResponse('Error code: ' + str(result.Status))


def verify(request, uuid):
    from aparnik.packages.shops.payments.views import payment as payment_view
    # order = get_object_or_404(Order.objects.active(), pk=request.POST.get('OrderId'))
    payment = get_object_or_404(Payment, uuid=uuid)

    token = request.POST.get('token')
    result_code = request.POST.get("ResCode")
    API_URL_REQUEST = 'https://sadad.shaparak.ir/vpg/api/v0/Advice/Verify'

    if payment.status != Payment.STATUS_WAITING:
        return payment_view(request, uuid=payment.uuid)

    if int(result_code) == 0:
        bank = get_object_or_404(Bank, authority_id=token)

        if bank.payment != payment:
            raise Http404

        verify_data = {
            'Token': token,
            'SignData': encrypt_des3(token),
        }

        init_response = requests.post(API_URL_REQUEST, json=verify_data)
        response_json = get_json(init_response)

        if response_json['ResCode'] == '0':
            # شماره پیگیری
            bank.ref_id = response_json['SystemTraceNo']

            # شماره مرجع
            bank.authority_id = response_json['RetrivalRefNo']

            bank.status = Bank.TRANSACTION_SUCCESS
            bank.save()
            payment.success()
            return payment_view(request, uuid=payment.uuid)

    payment.cancel()

    # return response
    return payment_view(request, uuid=payment.uuid)

def get_json(resp):
    """
    :param response:returned response as json when sending a request
    using 'requests' module.

    :return:response's content with json format
    """

    return json.loads(resp.content.decode('utf-8'))


def pad(text, pad_size=16):
    text_length = len(text)
    last_block_size = text_length % pad_size
    remaining_space = pad_size - last_block_size
    text = text + (remaining_space*chr(remaining_space))
    return text

def encrypt_des3(text):
    secret_key_bytes = base64.b64decode(Setting.objects.get(key='BANK_MELLI_SECRET_KEY').get_value())
    text = pad(text, 8)
    cipher = DES3.new(secret_key_bytes, DES3.MODE_ECB)
    my_export = cipher.encrypt(str.encode(text))
    return base64.b64encode(my_export).decode("utf-8")


# -*- coding: utf-8 -*-


from django.db import models
from django.db.models.signals import post_save
from aparnik.contrib.users.models import PhoneField
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse
from django.contrib.auth import get_user_model
from django.utils.text import Truncator

from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.notifications.models import Notification

User = get_user_model()


# Create your models here.
class ContactUsManager(BaseModelManager):
    def get_queryset(self):
        return super(ContactUsManager, self).get_queryset()

    def active(self):
        return self.get_queryset().filter(is_active=True)


class ContactUs(BaseModel):
    website = models.URLField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('Website')
    )

    address = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('Address')
    )

    phone = PhoneField(
        mobile=True,
        blank=True,
        null=True,
        verbose_name=_('Phone Number')
    )

    email = models.EmailField(
        null=True,
        blank=True,
        verbose_name=_('Email')
    )

    first_name = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        verbose_name=_('First name')
    )

    last_name = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        verbose_name=_('Last name')
    )

    title = models.TextField(
        blank=True,
        null=True,
        verbose_name=_('Title')
    )

    content = models.TextField(
        blank=True,
        null=True,
        verbose_name=_('Content'),
    )

    is_active = models.BooleanField(
        verbose_name=_('Is Active'),
        default=True
    )

    objects = ContactUsManager()

    def save(self, *args, **kwargs):

        return super(ContactUs, self).save(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        super(ContactUs, self).__init__(*args, **kwargs)

    def __str__(self):
        return '%s %s' % (self.id, self.title)

    class Meta:
        # ordering = ['latitude', 'longitude']
        verbose_name = _('Contact Us')
        verbose_name_plural = _('Contact Us')

    def get_api_uri(self):
        return reverse('aparnik-api:contactus:details', args=[self.id])

    def get_title(self):
        return self.title

    @property
    def fullname(self):
        """

        :return: fullname
        """
        return '{} {}'.format(self.first_name, self.last_name)


def post_save_constact_us_receiver(sender, instance, created, *args, **kwargs):
    if created:
        from .tasks import send_contact_us_message
        send_contact_us_message.delay(instance.pk)


post_save.connect(post_save_constact_us_receiver, sender=ContactUs)



from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer, FileFieldSummarySerializer
from aparnik.packages.shops.products.api.serializers import ProductListSerializer, ProductDetailSerializer
from ..models import File


# Base File List
class FileListSerializer(ProductListSerializer):
    url = serializers.SerializerMethodField()
    file_download_request_url = serializers.SerializerMethodField()
    banner = serializers.SerializerMethodField()
    cover = serializers.SerializerMethodField()
    file_property = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(FileListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = File
        fields = ProductListSerializer.Meta.fields + [
            'id',
            'url',
            'banner',
            'cover',
            'is_free',
            'file_download_request_url',
            'file_property',
        ]

        read_only_fields = ProductListSerializer.Meta.read_only_fields + ['url', 'file_download_request_url',
                                                                          'banner', 'cover', 'file_property',
                                                                          ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_file_download_request_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_file_download_request_uri())

    def get_banner(self, obj):
        return FileFieldListSerailizer(obj.banner, many=False, read_only=True, context=self.context).data if obj.banner else None

    def get_cover(self, obj):
        return FileFieldListSerailizer(obj.cover, many=False, read_only=True,
                                       context=self.context).data if obj.cover else None

    def get_file_property(self, obj):
        return FileFieldSummarySerializer(obj.file_obj, many=False, read_only=True,
                                       context=self.context).data if obj.file_obj else None


# Base File Details
class FileDetailSerializer(FileListSerializer, ProductDetailSerializer):

    def __init__(self, *args, **kwargs):
        super(FileDetailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = File
        fields = FileListSerializer.Meta.fields + ProductDetailSerializer.Meta.fields + [
            'description',

        ]
        read_only_fields = FileListSerializer.Meta.read_only_fields + ProductDetailSerializer.Meta.read_only_fields + [
            'description']


# Base File List Polymorphic
class FileListPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):

        model_serializer_mapping = {
            File: FileListSerializer,
        }

        # Book
        if is_app_installed('aparnik.packages.educations.books'):
            from aparnik.packages.educations.books.models import Book
            from aparnik.packages.educations.books.api.serializers import BookListSerializer
            model_serializer_mapping[Book] = BookListSerializer

        # Course File
        if is_app_installed('aparnik.packages.educations.courses'):
            from aparnik.packages.educations.courses.models import CourseFile
            from aparnik.packages.educations.courses.api.serializers import CourseFileListSerializer
            model_serializer_mapping[CourseFile] = CourseFileListSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(FileListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Base Model Details Polymorphic
class FileDetailsPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):

        model_serializer_mapping = {
            File: FileListSerializer,
        }

        # Book
        if is_app_installed('aparnik.packages.educations.books'):
            from aparnik.packages.educations.books.models import Book
            from aparnik.packages.educations.books.api.serializers import BookDetailsSerializer
            model_serializer_mapping[Book] = BookDetailsSerializer

        # Course File
        if is_app_installed('aparnik.packages.educations.courses'):
            from aparnik.packages.educations.courses.models import CourseFile
            from aparnik.packages.educations.courses.api.serializers import CourseFileDetailSerializer
            model_serializer_mapping[CourseFile] = CourseFileDetailSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(FileDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



# -*- coding: utf-8 -*-


from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class SettingsConfig(AppConfig):
    name = 'aparnik.contrib.settings'
    verbose_name = _('settings')
